# -*- coding: utf-8 -*-
""" convinience functions for dictionaries """
from __future__ import absolute_import, division, print_function, unicode_literals
import operator as op
import itertools as it
from collections import defaultdict, OrderedDict
from functools import partial
from six.moves import zip, range, map
from utool import util_inject
from utool import util_list
from utool import util_const
from utool import util_iter
import copy
import six
try:
    import numpy as np
    HAVE_NUMPY = True
except ImportError:
    HAVE_NUMPY = False
    pass
print, rrr, profile = util_inject.inject2(__name__)


@profile
def dzip(list1, list2):
    r"""
    Zips elementwise pairs between list1 and list2 into a dictionary. Values
    from list2 can be broadcast onto list1.

    Args:
        list1 (sequence): full sequence
        list2 (sequence): can either be a sequence of one item or a sequence of
            equal length to `list1`

    SeeAlso:
        util_list.broadcast_zip

    Returns:
        dict: similar to dict(zip(list1, list2))

    CommandLine:
        python -m utool.util_dict dzip

    Example:
        >>> # DISABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> assert dzip([1, 2, 3], [4]) == {1: 4, 2: 4, 3: 4}
        >>> assert dzip([1, 2, 3], [4, 4, 4]) == {1: 4, 2: 4, 3: 4}
        >>> ut.assert_raises(ValueError, dzip, [1, 2, 3], [])
        >>> ut.assert_raises(ValueError, dzip, [], [4, 5, 6])
        >>> ut.assert_raises(ValueError, dzip, [], [4])
        >>> ut.assert_raises(ValueError, dzip, [1, 2], [4, 5, 6])
        >>> ut.assert_raises(ValueError, dzip, [1, 2, 3], [4, 5])
    """
    try:
        len(list1)
    except TypeError:
        list1 = list(list1)
    try:
        len(list2)
    except TypeError:
        list2 = list(list2)
    if len(list1) == 0 and len(list2) == 1:
        # Corner case:
        # allow the first list to be empty and the second list to broadcast a
        # value. This means that the equality check wont work for the case
        # where list1 and list2 are supposed to correspond, but the length of
        # list2 is 1.
        list2 = []
    if len(list2) == 1 and len(list1) > 1:
        list2 = list2 * len(list1)
    if len(list1) != len(list2):
        raise ValueError('out of alignment len(list1)=%r, len(list2)=%r' % (
            len(list1), len(list2)))
    return dict(zip(list1, list2))


def map_dict_vals(func, dict_):
    """ applies a function to each of the keys in a dictionary

    Args:
        func (callable): a function
        dict_ (dict): a dictionary

    Returns:
        newdict: transformed dictionary

    CommandLine:
        python -m utool.util_dict --test-map_dict_vals

    Example:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> dict_ = {'a': [1, 2, 3], 'b': []}
        >>> func = len
        >>> newdict = map_dict_vals(func, dict_)
        >>> result = ut.repr2(newdict)
        >>> print(result)
        {'a': 3, 'b': 0}
    """
    if not hasattr(func, '__call__'):
        func = func.__getitem__
    keyval_list = [(key, func(val)) for key, val in six.iteritems(dict_)]
    dictclass = OrderedDict if isinstance(dict_, OrderedDict) else dict
    newdict = dictclass(keyval_list)
    # newdict = type(dict_)(keyval_list)
    return newdict


def map_dict_keys(func, dict_):
    """ applies a function to each of the keys in a dictionary

    Args:
        func (callable): a function
        dict_ (dict): a dictionary

    Returns:
        newdict: transformed dictionary

    CommandLine:
        python -m utool.util_dict --test-map_dict_keys

    Example:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> dict_ = {'a': [1, 2, 3], 'b': []}
        >>> func = ord
        >>> newdict = map_dict_keys(func, dict_)
        >>> result = ut.repr2(newdict)
        >>> ut.assert_raises(AssertionError, map_dict_keys, len, dict_)
        >>> print(result)
        {97: [1, 2, 3], 98: []}
    """
    if not hasattr(func, '__call__'):
        func = func.__getitem__
        # op.itemgetter(func)
    keyval_list = [(func(key), val) for key, val in six.iteritems(dict_)]
    # newdict = type(dict_)(keyval_list)
    dictclass = OrderedDict if isinstance(dict_, OrderedDict) else dict
    newdict = dictclass(keyval_list)
    assert len(newdict) == len(dict_), (
        'multiple input keys were mapped to the same output key')
    return newdict


map_vals = map_dict_vals
map_keys = map_dict_keys


class AutoVivification(dict):
    """
    Implementation of perl's autovivification feature.

    An AutoVivification is an infinitely nested default dict of dicts.

    References:
        http://stackoverflow.com/questions/651794/best-way-to-init-dict-of-dicts

    Example:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> dict_ = AutoVivification()
        >>> # Notice that there is no KeyError
        >>> dict_[0][10][100] = None
        >>> result = ('dict_ = %r' % (dict_,))
        >>> print(result)
        dict_ = {0: {10: {100: None}}}
    """
    def __getitem__(self, key):
        try:
            # value = super(AutoVivification, self).__getitem__(key)
            value = dict.__getitem__(self, key)
        except KeyError:
            value = self[key] = type(self)()
        return value


class OrderedAutoVivification(OrderedDict):
    """
    Implementation of perl's autovivification feature.

    An OrderedAutoVivification is an infinitely nested default dict of ordered
    dicts.

    References:
        http://stackoverflow.com/questions/651794/best-way-to-init-dict-of-dicts

    Doctest:
        >>> from utool.util_dict import *  # NOQA
        >>> dict_ = AutoOrderedDict()
        >>> # Notice that there is no KeyError
        >>> dict_[0][10][100] = None
        >>> dict_[0][10][1] = None
        >>> result = ('dict_ = %r' % (dict_,))
        >>> print(result)
        dict_ = {0: {10: {100: None, 1: None}}}
    """
    def __getitem__(self, key):
        try:
            # value = super(OrderedAutoVivification, self).__getitem__(key)
            value = OrderedDict.__getitem__(self, key)
        except KeyError:
            value = self[key] = type(self)()
        return value

    def __repr__(self):
        import utool as ut
        return ut.repr2(self)

    __str__ = __repr__

AutoDict = AutoVivification
AutoOrderedDict = OrderedAutoVivification


def count_dict_vals(dict_of_lists):
    count_dict = {'len(%s)' % (key,): len(val) for key, val in six.iteritems(dict_of_lists)}
    return count_dict


def dict_keysubset(dict_, keys):
    return [key for key in keys if key in dict_]


def get_dict_hashid(dict_):
    r"""
    Args:
        dict_ (dict):

    Returns:
        int: id hash

    References:
        http://stackoverflow.com/questions/5884066/hashing-a-python-dictionary

    CommandLine:
        python -m utool.util_dict --test-get_dict_hashid
        python3 -m utool.util_dict --test-get_dict_hashid

    Example:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> dict_ = {}
        >>> dict_ = {'a': 'b'}
        >>> dict_ = {'a': {'c': 'd'}}
        >>> #dict_ = {'a': {'c': 'd'}, 1: 143, dict: set}
        >>> #dict_ = {'a': {'c': 'd'}, 1: 143 } non-determenism
        >>> hashid = get_dict_hashid(dict_)
        >>> result = str(hashid)
        >>> print(result)
        mxgkepoboqjerkhb

        oegknoalkrkojumi
    """
    import utool as ut
    raw_text = ut.repr4(dict_, sorted_=True, strvals=True, nl=2)
    #print('raw_text = %r' % (raw_text,))
    hashid = ut.hashstr27(raw_text)
    #from utool import util_hash
    #hashid = hash(frozenset(dict_.items()))
    #hashid = util_hash.make_hash(dict_)
    return hashid


class hashdict(dict):
    """
    hashable dict implementation, suitable for use as a key into
    other dicts.

    Example:
        >>> # DISABLE_DOCTEST
        >>> h1 = hashdict({"apples": 1, "bananas":2})
        >>> h2 = hashdict({"bananas": 3, "mangoes": 5})
        >>> h1+h2
        hashdict(apples=1, bananas=3, mangoes=5)
        >>> d1 = {}
        >>> d1[h1] = "salad"
        >>> d1[h1]
        'salad'
        >>> d1[h2]
        Traceback (most recent call last):
        ...
        KeyError: hashdict(bananas=3, mangoes=5)

    References:
       http://stackoverflow.com/questions/1151658/python-hashable-dicts
       http://stackoverflow.com/questions/1151658/python-hashable-dicts
    """
    def __key(self):
        return tuple(sorted(self.items()))

    def __repr__(self):
        return "{0}({1})".format(self.__class__.__name__,
                                 ", ".join("{0}={1}".format(
                                     str(i[0]), repr(i[1])) for i in self.__key()))

    def __hash__(self):
        return hash(self.__key())
    def __setitem__(self, key, value):
        raise TypeError("{0} does not support item assignment"
                         .format(self.__class__.__name__))
    def __delitem__(self, key):
        raise TypeError("{0} does not support item assignment"
                         .format(self.__class__.__name__))
    def clear(self):
        raise TypeError("{0} does not support item assignment"
                         .format(self.__class__.__name__))
    def pop(self, *args, **kwargs):
        raise TypeError("{0} does not support item assignment"
                         .format(self.__class__.__name__))
    def popitem(self, *args, **kwargs):
        raise TypeError("{0} does not support item assignment"
                         .format(self.__class__.__name__))
    def setdefault(self, *args, **kwargs):
        raise TypeError("{0} does not support item assignment"
                         .format(self.__class__.__name__))
    def update(self, *args, **kwargs):
        raise TypeError("{0} does not support item assignment"
                         .format(self.__class__.__name__))
    # update is not ok because it mutates the object
    # __add__ is ok because it creates a new object
    # while the new object is under construction, it's ok to mutate it
    def __add__(self, right):
        result = hashdict(self)
        dict.update(result, right)
        return result


def dict_stack(dict_list, key_prefix=''):
    r"""
    stacks values from two dicts into a new dict where the values are list of
    the input values. the keys are the same.

    DEPRICATE in favor of dict_stack2

    Args:
        dict_list (list): list of dicts with similar keys

    Returns:
        dict dict_stacked

    CommandLine:
        python -m utool.util_dict --test-dict_stack
        python -m utool.util_dict --test-dict_stack:1

    Example:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> dict1_ = {'a': 1, 'b': 2}
        >>> dict2_ = {'a': 2, 'b': 3, 'c': 4}
        >>> dict_stacked = dict_stack([dict1_, dict2_])
        >>> result = ut.repr2(dict_stacked, sorted_=True)
        >>> print(result)
        {'a': [1, 2], 'b': [2, 3], 'c': [4]}

    Example1:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> # Get equivalent behavior with dict_stack2?
        >>> # Almost, as long as None is not part of the list
        >>> dict1_ = {'a': 1, 'b': 2}
        >>> dict2_ = {'a': 2, 'b': 3, 'c': 4}
        >>> dict_stacked_ = dict_stack2([dict1_, dict2_])
        >>> dict_stacked = {key: ut.filter_Nones(val) for key, val in dict_stacked_.items()}
        >>> result = ut.repr2(dict_stacked, sorted_=True)
        >>> print(result)
        {'a': [1, 2], 'b': [2, 3], 'c': [4]}
    """
    dict_stacked_ = defaultdict(list)
    for dict_ in dict_list:
        for key, val in six.iteritems(dict_):
            dict_stacked_[key_prefix + key].append(val)
    dict_stacked = dict(dict_stacked_)
    return dict_stacked


def dict_stack2(dict_list, key_suffix=None, default=None):
    """
    Stacks vals from a list of dicts into a dict of lists. Inserts Nones in
    place of empty items to preserve order.

    Args:
        dict_list (list): list of dicts
        key_suffix (str): (default = None)

    Returns:
        dict: stacked_dict

    Example:
        >>> # ENABLE_DOCTEST
        >>> # Usual case: multiple dicts as input
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> dict1_ = {'a': 1, 'b': 2}
        >>> dict2_ = {'a': 2, 'b': 3, 'c': 4}
        >>> dict_list = [dict1_, dict2_]
        >>> dict_stacked = dict_stack2(dict_list)
        >>> result = ut.repr2(dict_stacked)
        >>> print(result)
        {'a': [1, 2], 'b': [2, 3], 'c': [None, 4]}

    Example1:
        >>> # ENABLE_DOCTEST
        >>> # Corner case: one dict as input
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> dict1_ = {'a': 1, 'b': 2}
        >>> dict_list = [dict1_]
        >>> dict_stacked = dict_stack2(dict_list)
        >>> result = ut.repr2(dict_stacked)
        >>> print(result)
        {'a': [1], 'b': [2]}

    Example2:
        >>> # ENABLE_DOCTEST
        >>> # Corner case: zero dicts as input
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> dict_list = []
        >>> dict_stacked = dict_stack2(dict_list)
        >>> result = ut.repr2(dict_stacked)
        >>> print(result)
        {}

    Example3:
        >>> # ENABLE_DOCTEST
        >>> # Corner case: empty dicts as input
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> dict_list = [{}]
        >>> dict_stacked = dict_stack2(dict_list)
        >>> result = ut.repr2(dict_stacked)
        >>> print(result)
        {}

    Example4:
        >>> # ENABLE_DOCTEST
        >>> # Corner case: one dict is empty
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> dict1_ = {'a': [1, 2], 'b': [2, 3]}
        >>> dict2_ = {}
        >>> dict_list = [dict1_, dict2_]
        >>> dict_stacked = dict_stack2(dict_list)
        >>> result = ut.repr2(dict_stacked)
        >>> print(result)
        {'a': [[1, 2], None], 'b': [[2, 3], None]}

    Example5:
        >>> # ENABLE_DOCTEST
        >>> # Corner case: disjoint dicts
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> dict1_ = {'a': [1, 2], 'b': [2, 3]}
        >>> dict2_ = {'c': 4}
        >>> dict_list = [dict1_, dict2_]
        >>> dict_stacked = dict_stack2(dict_list)
        >>> result = ut.repr2(dict_stacked)
        >>> print(result)
        {'a': [[1, 2], None], 'b': [[2, 3], None], 'c': [None, 4]}

    Example6:
        >>> # ENABLE_DOCTEST
        >>> # Corner case: 3 dicts
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> dict_list = [{'a': 1}, {'b': 1}, {'c': 1}, {'b': 2}]
        >>> default = None
        >>> dict_stacked = dict_stack2(dict_list, default=default)
        >>> result = ut.repr2(dict_stacked)
        >>> print(result)
        {'a': [1, None, None, None], 'b': [None, 1, None, 2], 'c': [None, None, 1, None]}
    """
    if len(dict_list) > 0:
        dict_list_ = [map_dict_vals(lambda x: [x], kw) for kw in dict_list]
        # Reduce does not handle default quite correctly
        default1 = []
        default2 = [default]
        accum_ = dict_list_[0]
        for dict_ in dict_list_[1:]:
            default1.append(default)
            accum_ = dict_union_combine(accum_, dict_, default=default1,
                                        default2=default2)
        stacked_dict = accum_
        # stacked_dict = reduce(partial(dict_union_combine, default=[default]), dict_list_)
    else:
        stacked_dict = {}
    # Augment keys if requested
    if key_suffix is not None:
        stacked_dict = map_dict_keys(lambda x: x + key_suffix, stacked_dict)
    return stacked_dict


def invert_dict(dict_, unique_vals=True):
    """
    Reverses the keys and values in a dictionary. Set unique_vals to False if
    the values in the dict are not unique.

    Args:
        dict_ (dict_): dictionary
        unique_vals (bool): if False, inverted keys are returned in a list.

    Returns:
        dict: inverted_dict

    CommandLine:
        python -m utool.util_dict --test-invert_dict

    Example:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> dict_ = {'a': 1, 'b': 2}
        >>> inverted_dict = invert_dict(dict_)
        >>> result = ut.repr4(inverted_dict, nl=False)
        >>> print(result)
        {1: 'a', 2: 'b'}

    Example:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> dict_ = OrderedDict([(2, 'good',), (1, 'ok',), (0, 'junk',), (None, 'UNKNOWN',)])
        >>> inverted_dict = invert_dict(dict_)
        >>> result = ut.repr4(inverted_dict, nl=False)
        >>> print(result)
        {'good': 2, 'ok': 1, 'junk': 0, 'UNKNOWN': None}

    Example:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> dict_ = {'a': 1, 'b': 0, 'c': 0, 'd': 0, 'e': 0, 'f': 2}
        >>> inverted_dict = invert_dict(dict_, unique_vals=False)
        >>> inverted_dict = ut.map_dict_vals(sorted, inverted_dict)
        >>> result = ut.repr4(inverted_dict, nl=False)
        >>> print(result)
        {0: ['b', 'c', 'd', 'e'], 1: ['a'], 2: ['f']}
    """
    if unique_vals:
        inverted_items = [(val, key) for key, val in six.iteritems(dict_)]
        inverted_dict = type(dict_)(inverted_items)
    else:
        inverted_dict = group_items(dict_.keys(), dict_.values())
    return inverted_dict


def iter_all_dict_combinations_ordered(varied_dict):
    """
    Same as all_dict_combinations but preserves order
    """
    tups_list = [[(key, val) for val in val_list]
                 for (key, val_list) in six.iteritems(varied_dict)]
    dict_iter = (OrderedDict(tups) for tups in it.product(*tups_list))
    return dict_iter


def all_dict_combinations_ordered(varied_dict):
    """
    Same as all_dict_combinations but preserves order
    """
    dict_list = list(iter_all_dict_combinations_ordered)
    return dict_list


def all_dict_combinations(varied_dict):
    """
    all_dict_combinations

    Args:
        varied_dict (dict):  a dict with lists of possible parameter settings

    Returns:
        list: dict_list a list of dicts correpsonding to all combinations of params settings

    CommandLine:
        python -m utool.util_dict --test-all_dict_combinations

    Example:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> varied_dict = {'logdist_weight': [0.0, 1.0], 'pipeline_root': ['vsmany'], 'sv_on': [True, False, None]}
        >>> dict_list = all_dict_combinations(varied_dict)
        >>> result = str(ut.repr4(dict_list))
        >>> print(result)
        [
            {'logdist_weight': 0.0, 'pipeline_root': 'vsmany', 'sv_on': True},
            {'logdist_weight': 0.0, 'pipeline_root': 'vsmany', 'sv_on': False},
            {'logdist_weight': 0.0, 'pipeline_root': 'vsmany', 'sv_on': None},
            {'logdist_weight': 1.0, 'pipeline_root': 'vsmany', 'sv_on': True},
            {'logdist_weight': 1.0, 'pipeline_root': 'vsmany', 'sv_on': False},
            {'logdist_weight': 1.0, 'pipeline_root': 'vsmany', 'sv_on': None},
        ]
    """
    #tups_list = [[(key, val) for val in val_list]
    #             if isinstance(val_list, (list, tuple))
    #             else [(key, val_list)]
    #             for (key, val_list) in six.iteritems(varied_dict)]
    tups_list = [[(key, val) for val in val_list]
                 if isinstance(val_list, (list))
                 #if isinstance(val_list, (list, tuple))
                 else [(key, val_list)]
                 for (key, val_list) in iteritems_sorted(varied_dict)]
    dict_list = [dict(tups) for tups in it.product(*tups_list)]
    #dict_list = [{key: val for (key, val) in tups} for tups in it.product(*tups_list)]
    #from collections import OrderedDict
    #dict_list = [OrderedDict([(key, val) for (key, val) in tups]) for tups in it.product(*tups_list)]
    return dict_list


def all_dict_combinations_lbls(varied_dict, remove_singles=True, allow_lone_singles=False):
    """
    returns a label for each variation in a varydict.

    It tries to not be oververbose and returns only what parameters are varied
    in each label.

    CommandLine:
        python -m utool.util_dict --test-all_dict_combinations_lbls
        python -m utool.util_dict --exec-all_dict_combinations_lbls:1

    Example:
        >>> # ENABLE_DOCTEST
        >>> import utool
        >>> from utool.util_dict import *  # NOQA
        >>> varied_dict = {'logdist_weight': [0.0, 1.0], 'pipeline_root': ['vsmany'], 'sv_on': [True, False, None]}
        >>> comb_lbls = utool.all_dict_combinations_lbls(varied_dict)
        >>> result = (utool.repr4(comb_lbls))
        >>> print(result)
        [
            'logdist_weight=0.0,sv_on=True',
            'logdist_weight=0.0,sv_on=False',
            'logdist_weight=0.0,sv_on=None',
            'logdist_weight=1.0,sv_on=True',
            'logdist_weight=1.0,sv_on=False',
            'logdist_weight=1.0,sv_on=None',
        ]

    Example:
        >>> # ENABLE_DOCTEST
        >>> import utool as ut
        >>> from utool.util_dict import *  # NOQA
        >>> varied_dict = {'logdist_weight': [0.0], 'pipeline_root': ['vsmany'], 'sv_on': [True]}
        >>> allow_lone_singles = True
        >>> comb_lbls = ut.all_dict_combinations_lbls(varied_dict, allow_lone_singles=allow_lone_singles)
        >>> result = (ut.repr4(comb_lbls))
        >>> print(result)
        [
            'logdist_weight=0.0,pipeline_root=vsmany,sv_on=True',
        ]
    """
    is_lone_single = all([
        isinstance(val_list, (list, tuple)) and len(val_list) == 1
        for key, val_list in iteritems_sorted(varied_dict)
    ])
    if not remove_singles or (allow_lone_singles and is_lone_single):
        # all entries have one length
        multitups_list = [
            [(key, val) for val in val_list]
            for key, val_list in iteritems_sorted(varied_dict)
        ]
    else:
        multitups_list = [
            [(key, val) for val in val_list]
            for key, val_list in iteritems_sorted(varied_dict)
            if isinstance(val_list, (list, tuple)) and len(val_list) > 1]
    combtup_list = list(it.product(*multitups_list))
    combtup_list2 = [
        [(key, val) if isinstance(val, six.string_types) else (key, repr(val))
         for (key, val) in combtup]
        for combtup in combtup_list]
    comb_lbls = [','.join(['%s=%s' % (key, val) for (key, val) in combtup])
                 for combtup in combtup_list2]
    #comb_lbls = list(map(str, comb_pairs))
    return comb_lbls


def dict_union2(dict1, dict2):
    return dict(list(dict1.items()) + list(dict2.items()))


def dict_union(*args):
    return dict([item for dict_ in iter(args) for item in six.iteritems(dict_)])


def build_conflict_dict(key_list, val_list):
    """
    Builds dict where a list of values is associated with more than one key

    Args:
        key_list (list):
        val_list (list):

    Returns:
        dict: key_to_vals

    CommandLine:
        python -m utool.util_dict --test-build_conflict_dict

    Example:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> key_list = [  1,   2,   2,   3,   1]
        >>> val_list = ['a', 'b', 'c', 'd', 'e']
        >>> key_to_vals = build_conflict_dict(key_list, val_list)
        >>> result = ut.repr4(key_to_vals)
        >>> print(result)
        {
            1: ['a', 'e'],
            2: ['b', 'c'],
            3: ['d'],
        }
    """
    key_to_vals = defaultdict(list)
    for key, val in zip(key_list, val_list):
        key_to_vals[key].append(val)
    return key_to_vals


def assert_keys_are_subset(dict1, dict2):
    """
    Example:
        >>> # DISABLE_DOCTEST
        >>> dict1 = {1:1, 2:2, 3:3}
        >>> dict2 = {2:3, 3:3}
        >>> assert_keys_are_subset(dict1, dict2)
        >>> #dict2 = {4:3, 3:3}
    """
    keys1 = set(dict1.keys())
    keys2 = set(dict2.keys())
    unknown_keys = keys2.difference(keys1)
    assert len(unknown_keys) == 0, 'unknown_keys=%r' % (unknown_keys,)


def augdict(dict1, dict2=None, **kwargs):
    dict1_ = copy.deepcopy(dict1)
    if dict2 is not None:
        dict1_ = update_existing(dict1_, dict2, assert_exists=True)
    if len(kwargs) > 0:
        dict1_ = update_existing(dict1_, kwargs, assert_exists=True)
    return dict1_


def update_existing(dict1, dict2, copy=False, assert_exists=False,
                    iswarning=False, alias_dict=None):
    r"""
    updates vals in dict1 using vals from dict2 only if the
    key is already in dict1.

    Args:
        dict1 (dict):
        dict2 (dict):
        copy (bool): if true modifies dictionary in place (default = False)
        assert_exists (bool): if True throws error if new key specified (default = False)
        alias_dict (dict): dictionary of alias keys for dict2 (default = None)

    Returns:
        dict - updated dictionary

    CommandLine:
        python -m utool.util_dict --test-update_existing

    Example:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> dict1 = {'a': 1, 'b': 2, 'c': 3}
        >>> dict2 = {'a': 2, 'd': 3}
        >>> dict1_ = update_existing(dict1, dict2)
        >>> assert 'd' not in dict1
        >>> assert dict1['a'] == 2
        >>> assert dict1_ is dict1
    """
    if assert_exists:
        try:
            assert_keys_are_subset(dict1, dict2)
        except AssertionError as ex:
            from utool import util_dbg
            util_dbg.printex(ex, iswarning=iswarning, N=1)
            if not iswarning:
                raise
    if copy:
        dict1 = dict(dict1)
    if alias_dict is None:
        alias_dict = {}
    for key, val in six.iteritems(dict2):
        key = alias_dict.get(key, key)
        if key in dict1:
            dict1[key] = val
    return dict1


def update_dict(dict1, dict2, copy=False, alias_dict=None):
    if copy:
        dict1 = dict(dict1)
    if alias_dict is None:
        alias_dict = {}
    for key, val in six.iteritems(dict2):
        key = alias_dict.get(key, key)
        dict1[key] = val
    return dict1


def dict_update_newkeys(dict_, dict2):
    """ Like dict.update, but does not overwrite items """
    for key, val in six.iteritems(dict2):
        if key not in dict_:
            dict_[key] = val


def is_dicteq(dict1_, dict2_, almosteq_ok=True, verbose_err=True):
    """ Checks to see if dicts are the same. Performs recursion. Handles numpy """
    import utool as ut
    assert len(dict1_) == len(dict2_), 'dicts are not of same length'
    try:
        for (key1, val1), (key2, val2) in zip(dict1_.items(), dict2_.items()):
            assert key1 == key2, 'key mismatch'
            assert type(val1) == type(val2), 'vals are not same type'
            if HAVE_NUMPY and np.iterable(val1):
                if almosteq_ok and ut.is_float(val1):
                    assert np.all(ut.almost_eq(val1, val2)), 'float vals are not within thresh'
                else:
                    assert all([np.all(x1 == x2) for (x1, x2) in zip(val1, val2)]), 'np vals are different'
            elif isinstance(val1, dict):
                is_dicteq(val1, val2, almosteq_ok=almosteq_ok, verbose_err=verbose_err)
            else:
                assert val1 == val2, 'vals are different'
    except AssertionError as ex:
        if verbose_err:
            ut.printex(ex)
        return False
    return True


def dict_subset(dict_, keys, default=util_const.NoParam):
    r"""
    Args:
        dict_ (dict):
        keys (list):

    Returns:
        dict: subset dictionary

    Example:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> dict_ = {'K': 3, 'dcvs_clip_max': 0.2, 'p': 0.1}
        >>> keys = ['K', 'dcvs_clip_max']
        >>> d = tuple([])
        >>> subdict_ = dict_subset(dict_, keys)
        >>> result = ut.repr4(subdict_, sorted_=True, newlines=False)
        >>> print(result)
        {'K': 3, 'dcvs_clip_max': 0.2}
    """
    if default is util_const.NoParam:
        items = dict_take(dict_, keys)
    else:
        items = dict_take(dict_, keys, default)
    subdict_ = OrderedDict(list(zip(keys, items)))
    #item_sublist = [(key, dict_[key]) for key in keys]
    ##subdict_ = type(dict_)(item_sublist)  # maintain old dict format
    #subdict_ = OrderedDict(item_sublist)
    return subdict_


def dict_to_keyvals(dict_):
    return list(six.iteritems(dict_))


def dict_setdiff(dict_, negative_keys):
    r"""
    returns a copy of dict_ without keys in the negative_keys list

    Args:
        dict_ (dict):
        negative_keys (list):
    """
    keys = [key for key in six.iterkeys(dict_)
            if key not in set(negative_keys)]
    subdict_ = dict_subset(dict_, keys)
    return subdict_


def delete_dict_keys(dict_, key_list):
    r"""
    Removes items from a dictionary inplace. Keys that do not exist are
    ignored.

    Args:
        dict_ (dict): dict like object with a __del__ attribute
        key_list (list): list of keys that specify the items to remove

    CommandLine:
        python -m utool.util_dict --test-delete_dict_keys

    Example:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> dict_ = {'bread': 1, 'churches': 1, 'cider': 2, 'very small rocks': 2}
        >>> key_list = ['duck', 'bread', 'cider']
        >>> delete_dict_keys(dict_, key_list)
        >>> result = ut.repr4(dict_, nl=False)
        >>> print(result)
        {'churches': 1, 'very small rocks': 2}

    """
    invalid_keys = set(key_list) - set(dict_.keys())
    valid_keys = set(key_list) - invalid_keys
    for key in valid_keys:
        del dict_[key]
    return dict_


delete_keys = delete_dict_keys


def dict_take_gen(dict_, keys, *d):
    r"""
    generate multiple values from a dictionary

    Args:
        dict_ (dict):
        keys (list):

    Varargs:
        d: if specified is default for key errors

    CommandLine:
        python -m utool.util_dict --test-dict_take_gen

    Example1:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> dict_ = {1: 'a', 2: 'b', 3: 'c'}
        >>> keys = [1, 2, 3, 4, 5]
        >>> result = list(dict_take_gen(dict_, keys, None))
        >>> result = ut.repr4(result, nl=False)
        >>> print(result)
        ['a', 'b', 'c', None, None]

    Example2:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> dict_ = {1: 'a', 2: 'b', 3: 'c'}
        >>> keys = [1, 2, 3, 4, 5]
        >>> try:
        >>>     print(list(dict_take_gen(dict_, keys)))
        >>>     result = 'did not get key error'
        >>> except KeyError:
        >>>     result = 'correctly got key error'
        >>> print(result)
        correctly got key error
    """
    if isinstance(keys, six.string_types):
        # hack for string keys that makes copy-past easier
        keys = keys.split(', ')
    if len(d) == 0:
        # no default given throws key error
        dictget = dict_.__getitem__
    elif len(d) == 1:
        # default given does not throw key erro
        dictget = dict_.get
    else:
        raise ValueError('len(d) must be 1 or 0')
    for key in keys:
        if HAVE_NUMPY and isinstance(key, np.ndarray):
            # recursive call
            yield list(dict_take_gen(dict_, key, *d))
        else:
            yield dictget(key, *d)


def dict_take(dict_, keys, *d):
    """ get multiple values from a dictionary """
    try:
        return list(dict_take_gen(dict_, keys, *d))
    except TypeError:
        return list(dict_take_gen(dict_, keys, *d))[0]
    #return [dict_[key] for key in keys]

dict_take_list = dict_take
#def dict_take(dict_, keys, *d):
#    """ alias """
#    try:
#        return dict_take_list(dict_, keys, *d)
#    except TypeError:
#        return dict_take_list(dict_, [keys], *d)[0]


#def dict_unflat_take(dict_, unflat_key_list, *d):
#    return [dict_unflat_take(dict_, xs, *d)
#            if isinstance(xs, list) else
#            dict_take(dict_, xs, *d)
#            for xs in unflat_key_list]


def dict_take_asnametup(dict_, keys, name='_NamedTup'):
    from collections import namedtuple
    values = dict_take(dict_, keys)
    _NamedTup = namedtuple(name, keys)
    tup = _NamedTup(*values)
    return tup


def dict_take_pop(dict_, keys, *d):
    """ like dict_take but pops values off

    CommandLine:
        python -m utool.util_dict --test-dict_take_pop

    Example1:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> dict_ = {1: 'a', 'other': None, 'another': 'foo', 2: 'b', 3: 'c'}
        >>> keys = [1, 2, 3, 4, 5]
        >>> print('before: ' + ut.repr4(dict_))
        >>> result = list(dict_take_pop(dict_, keys, None))
        >>> result = ut.repr4(result, nl=False)
        >>> print('after: ' + ut.repr4(dict_))
        >>> assert len(dict_) == 2
        >>> print(result)
        ['a', 'b', 'c', None, None]

    Example2:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> dict_ = {1: 'a', 2: 'b', 3: 'c'}
        >>> keys = [1, 2, 3, 4, 5]
        >>> print('before: ' + ut.repr4(dict_))
        >>> try:
        >>>     print(list(dict_take_pop(dict_, keys)))
        >>>     result = 'did not get key error'
        >>> except KeyError:
        >>>     result = 'correctly got key error'
        >>> assert len(dict_) == 0
        >>> print('after: ' + ut.repr4(dict_))
        >>> print(result)
        correctly got key error
    """
    if len(d) == 0:
        return [dict_.pop(key) for key in keys]
    elif len(d) == 1:
        default = d[0]
        return [dict_.pop(key, default) for key in keys]
    else:
        raise ValueError('len(d) must be 1 or 0')


def dict_assign(dict_, keys, vals):
    """ simple method for assigning or setting values with a similar interface
    to dict_take """
    for key, val in zip(keys, vals):
        dict_[key] = val


def dict_where_len0(dict_):
    """
    Accepts a dict of lists. Returns keys that have vals with no length
    """
    keys = np.array(dict_.keys())
    flags = np.array(list(map(len, dict_.values()))) == 0
    indices = np.where(flags)[0]
    return keys[indices]


def get_dict_column(dict_, colx):
    r"""
    Args:
        dict_ (dict_): a dictionary of lists
        colx (int):

    CommandLine:
        python -m utool.util_dict --test-get_dict_column

    Example:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> dict_ = {'a': [0, 1, 2], 'b': [3, 4, 5], 'c': [6, 7, 8]}
        >>> colx = [2, 0]
        >>> retdict_ = get_dict_column(dict_, colx)
        >>> result = ut.repr2(retdict_)
        >>> print(result)
        {'a': [2, 0], 'b': [5, 3], 'c': [8, 6]}
    """
    retdict_ = {key: util_list.list_take(val, colx)
                for key, val in six.iteritems(dict_)}
    return retdict_


def dict_take_column(list_of_dicts_, colkey, default=None):
    return [dict_.get(colkey, default) for dict_ in list_of_dicts_]


def dict_set_column(list_of_dicts_, colkey, value_list):
    for dict_, value in zip(list_of_dicts_, value_list):
        dict_[colkey] = value


def dictinfo(dict_):
    """
    dictinfo

    In depth debugging info

    Args:
        dict_ (dict):

    Returns:
        str

    Example:
        >>> # DISABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> dict_ = {}
        >>> result = dictinfo(dict_)
        >>> print(result)
    """
    import utool as ut
    if not isinstance(dict_, dict):
        return 'expected dict got %r' % type(dict_)

    keys = list(dict_.keys())
    vals = list(dict_.values())
    num_keys  = len(keys)
    key_types = list(set(map(type, keys)))
    val_types = list(set(map(type, vals)))

    fmtstr_ = '\n' + ut.unindent('''
    * num_keys  = {num_keys}
    * key_types = {key_types}
    * val_types = {val_types}
    '''.strip('\n'))

    if len(val_types) == 1:
        if val_types[0] == np.ndarray:
            # each key holds an ndarray
            val_shape_stats = ut.get_stats(set(map(np.shape, vals)), axis=0)
            val_shape_stats_str = ut.repr4(val_shape_stats, strvals=True, newlines=False)
            val_dtypes = list(set([val.dtype for val in vals]))
            fmtstr_ += ut.unindent('''
            * val_shape_stats = {val_shape_stats_str}
            * val_dtypes = {val_dtypes}
            '''.strip('\n'))
        elif val_types[0] == list:
            # each key holds a list
            val_len_stats =  ut.get_stats(set(map(len, vals)))
            val_len_stats_str = ut.repr4(val_len_stats, strvals=True, newlines=False)
            depth = ut.list_depth(vals)
            deep_val_types = list(set(ut.list_deep_types(vals)))
            fmtstr_ += ut.unindent('''
            * list_depth = {depth}
            * val_len_stats = {val_len_stats_str}
            * deep_types = {deep_val_types}
            '''.strip('\n'))
            if len(deep_val_types) == 1:
                if deep_val_types[0] == np.ndarray:
                    deep_val_dtypes = list(set([val.dtype for val in vals]))
                    fmtstr_ += ut.unindent('''
                    * deep_val_dtypes = {deep_val_dtypes}
                    ''').strip('\n')
        elif val_types[0] in [np.uint8, np.int8, np.int32, np.int64, np.float16, np.float32, np.float64]:
            # each key holds a scalar
            val_stats = ut.get_stats(vals)
            fmtstr_ += ut.unindent('''
            * val_stats = {val_stats}
            ''').strip('\n')

    fmtstr = fmtstr_.format(**locals())
    return ut.indent(fmtstr)


def dict_find_keys(dict_, val_list):
    r"""
    Args:
        dict_ (dict):
        val_list (list):

    Returns:
        dict: found_dict

    CommandLine:
        python -m utool.util_dict --test-dict_find_keys

    Example:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> dict_ = {'default': 1, 'hierarchical': 5, 'linear': 0, 'kdtree': 1,
        ...          'composite': 3, 'autotuned': 255, 'saved': 254, 'kmeans': 2,
        ...          'lsh': 6, 'kdtree_single': 4}
        >>> val_list = [1]
        >>> found_dict = dict_find_keys(dict_, val_list)
        >>> result = ut.repr2(ut.map_vals(sorted, found_dict))
        >>> print(result)
        {1: ['default', 'kdtree']}
    """
    found_dict = {
        search_val: [key for key, val in six.iteritems(dict_)
                     if val == search_val]
        for search_val in val_list
    }
    return found_dict


def dict_find_other_sameval_keys(dict_, key):
    """
    Example:
        >>> # DISABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> dict_ = {'default': 1, 'hierarchical': 5, 'linear': 0, 'kdtree': 1,
        ...          'composite': 3, 'autotuned': 255, 'saved': 254, 'kmeans': 2,
        ...          'lsh': 6, 'kdtree_single': 4}
        >>> key = 'default'
        >>> found_dict = dict_find_keys(dict_, val_list)
    """
    value = dict_[key]
    found_dict = dict_find_keys(dict_, [value])
    other_keys = found_dict[value]
    other_keys.remove(key)
    return other_keys


@profile
def dict_hist(item_list, weight_list=None, ordered=False, labels=None):
    r"""
    Builds a histogram of items in item_list

    Args:
        item_list (list): list with hashable items (usually containing duplicates)

    Returns:
        dict : dictionary where the keys are items in item_list, and the values
          are the number of times the item appears in item_list.

    CommandLine:
        python -m utool.util_dict --test-dict_hist

    Example:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> item_list = [1, 2, 39, 900, 1232, 900, 1232, 2, 2, 2, 900]
        >>> hist_ = dict_hist(item_list)
        >>> result = ut.repr2(hist_)
        >>> print(result)
        {1: 1, 2: 4, 39: 1, 900: 3, 1232: 2}
    """
    if labels is None:
        # hist_ = defaultdict(lambda: 0)
        hist_ = defaultdict(int)
    else:
        hist_ = {k: 0 for k in labels}
    if weight_list is None:
        # weight_list = it.repeat(1)
        for item in item_list:
            hist_[item] += 1
    else:
        for item, weight in zip(item_list, weight_list):
            hist_[item] += weight
    # hist_ = dict(hist_)
    if ordered:
        # import utool as ut
        # key_order = ut.sortedby(list(hist_.keys()), list(hist_.values()))
        getval = op.itemgetter(1)
        key_order = [key for (key, value) in sorted(hist_.items(), key=getval)]
        hist_ = order_dict_by(hist_, key_order)
    return hist_


def range_hist(items, bins):
    """
    Bins items into a discrete histogram by values and/or ranges.

        items = [1, 2, 3, 4, 5, 6, 7]
        bins = [0, 1, 2, (3, float('inf'))]
        ut.range_hist(items, bins)
    """
    big_hist = ut.dict_hist(items)
    hist = ut.odict([(b, 0) for b in bins])

    for k, v in big_hist.items():
        for b in bins:
            if isinstance(b, (list, tuple)):
                if k >= b[0] and k < b[1]:
                    hist[b] += v
            elif k == b:
                hist[b] += v
    return hist


def dict_hist_cumsum(hist_, reverse=True):
    """ VERY HACKY """
    import utool as ut
    items = hist_.items()
    if reverse:
        items = sorted(items)[::-1]
    else:
        items = sorted(items)
    key_list = ut.get_list_column(items, 0)
    val_list = ut.get_list_column(items, 1)
    cumhist_ = dict(zip(key_list, np.cumsum(val_list)))
    return cumhist_


def merge_dicts(*args):
    r"""
    add / concatenate / union / join / merge / combine dictionaries

    Copies the first dictionary given and then repeatedly calls update using
    the rest of the dicts given in args. Duplicate keys will receive the last
    value specified the list of dictionaries.

    Returns:
        dict: mergedict_

    CommandLine:
        python -m utool.util_dict --test-merge_dicts

    References:
        http://stackoverflow.com/questions/38987/how-can-i-merge-two-python-dictionaries-in-a-single-expression

    Example:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> x = {'a': 1, 'b': 2}
        >>> y = {'b': 3, 'c': 4}
        >>> mergedict_ = merge_dicts(x, y)
        >>> result = ut.repr4(mergedict_, sorted_=True, newlines=False)
        >>> print(result)
        {'a': 1, 'b': 3, 'c': 4}

    """
    iter_ = iter(args)
    mergedict_ = six.next(iter_).copy()
    for dict_ in iter_:
        mergedict_.update(dict_)
    return mergedict_


def dict_union3(dict1, dict2, combine_op=op.add):
    r"""
    Args:
        dict1 (dict):
        dict2 (dict):
        combine_op (func): (default=op.add)

    Returns:
        dict: mergedict_

    CommandLine:
        python -m utool.util_dict --exec-dict_union3

    Example:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> dict1 = {'a': 1, 'b': 2, 'c': 3, 'd': 4}
        >>> dict2 = {'b': 2, 'c': 3, 'd': 5, 'e': 21, 'f': 42}
        >>> combine_op = op.add
        >>> mergedict_ = dict_union3(dict1, dict2, combine_op)
        >>> result = ('mergedict_ = %s' % (ut.repr4(mergedict_, nl=False),))
        >>> print(result)
        mergedict_ = {'a': 1, 'b': 4, 'c': 6, 'd': 9, 'e': 21, 'f': 42}
    """
    keys1 = set(dict1.keys())
    keys2 = set(dict2.keys())
    # Combine common keys
    keys3 = keys1.intersection(keys2)
    if len(keys3) > 0 and combine_op is None:
        raise AssertionError('Can only combine disjoint dicts when combine_op is None')
    dict3 = {key: combine_op(dict1[key], dict2[key]) for key in keys3}
    # Combine unique keys
    for key in keys1.difference(keys3):
        dict3[key] = dict1[key]
    for key in keys2.difference(keys3):
        dict3[key] = dict2[key]
    return dict3


def dict_intersection(dict1, dict2, combine=False, combine_op=op.add):
    r"""
    Args:
        dict1 (dict):
        dict2 (dict):
        combine (bool): Combines keys only if the values are equal if False else
            values are combined using combine_op (default = False)
        combine_op (func): (default = op.add)

    Returns:
        dict: mergedict_

    CommandLine:
        python -m utool.util_dict --exec-dict_intersection

    Example:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> dict1 = {'a': 1, 'b': 2, 'c': 3, 'd': 4}
        >>> dict2 = {'b': 2, 'c': 3, 'd': 5, 'e': 21, 'f': 42}
        >>> combine = False
        >>> mergedict_ = dict_intersection(dict1, dict2, combine)
        >>> result = ('mergedict_ = %s' % (ut.repr4(mergedict_, nl=False),))
        >>> print(result)
        mergedict_ = {'b': 2, 'c': 3}
    """
    isect_keys = set(dict1.keys()).intersection(set(dict2.keys()))
    if combine:
        # TODO: depricate this
        dict_isect = {k: combine_op(dict1[k], dict2[k]) for k in isect_keys}
    else:
        # maintain order if possible
        if isinstance(dict1, OrderedDict):
            isect_keys_ = [k for k in dict1.keys() if k in isect_keys]
            _dict_cls = OrderedDict
        else:
            isect_keys_ = isect_keys
            _dict_cls = dict
        dict_isect = _dict_cls(
            (k, dict1[k]) for k in isect_keys_ if dict1[k] == dict2[k]
        )
    return dict_isect


def dict_isect_combine(dict1, dict2, combine_op=op.add):
    """ Intersection of dict keys and combination of dict values """
    keys3 = set(dict1.keys()).intersection(set(dict2.keys()))
    dict3 = {key: combine_op(dict1[key], dict2[key]) for key in keys3}
    return dict3


def dict_union_combine(dict1, dict2, combine_op=op.add,
                       default=util_const.NoParam,
                       default2=util_const.NoParam):
    """
    Combine of dict keys and uses dfault value when key does not exist

    CAREFUL WHEN USING THIS WITH REDUCE. Use dict_stack2 instead
    """
    keys3 = set(dict1.keys()).union(set(dict2.keys()))
    if default is util_const.NoParam:
        dict3 = {key: combine_op(dict1[key], dict2[key]) for key in keys3}
    else:
        if default2 is util_const.NoParam:
            default2 = default
        dict3 = {key: combine_op(dict1.get(key, default), dict2.get(key, default2))
                 for key in keys3}
    return dict3


def dict_accum(*dict_list):
    accumulator = defaultdict(list)
    for dict_ in dict_list:
        for key, val in dict_.items():
            accumulator[key].append(val)
    return accumulator

dict_isect = dict_intersection


def dict_filter_nones(dict_):
    r"""
    Removes None values

    Args:
        dict_ (dict):  a dictionary

    Returns:
        dict:

    CommandLine:
        python -m utool.util_dict --exec-dict_filter_nones

    Example:
        >>> # DISABLE_DOCTEST
        >>> # UNSTABLE_DOCTEST
        >>> # fails on python 3 because of dict None order
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> dict_ = {1: None, 2: 'blue', 3: 'four', None: 'fun'}
        >>> dict2_ = dict_filter_nones(dict_)
        >>> result = ut.repr4(dict2_, nl=False)
        >>> print(result)
        {None: 'fun', 2: 'blue', 3: 'four'}
    """
    dict2_ = {
        key: val
        for key, val in six.iteritems(dict_)
        if val is not None
    }
    return dict2_


def groupby_tags(item_list, tags_list):
    r"""
    case where an item can belong to multiple groups

    Args:
        item_list (list):
        tags_list (list):

    Returns:
        dict: groupid_to_items

    CommandLine:
        python -m utool.util_dict --test-groupby_tags

    Example:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> tagged_item_list = {
        >>>     'spam': ['meat', 'protein', 'food'],
        >>>     'eggs': ['protein', 'food'],
        >>>     'cheese': ['dairy', 'protein', 'food'],
        >>>     'jam':  ['fruit', 'food'],
        >>>     'banana': ['weapon', 'fruit', 'food'],
        >>> }
        >>> item_list = list(tagged_item_list.keys())
        >>> tags_list = list(tagged_item_list.values())
        >>> groupid_to_items = groupby_tags(item_list, tags_list)
        >>> groupid_to_items = ut.map_vals(sorted, groupid_to_items)
        >>> result = ('groupid_to_items = %s' % (ut.repr4(groupid_to_items),))
        >>> print(result)
        groupid_to_items = {
            'dairy': ['cheese'],
            'food': ['banana', 'cheese', 'eggs', 'jam', 'spam'],
            'fruit': ['banana', 'jam'],
            'meat': ['spam'],
            'protein': ['cheese', 'eggs', 'spam'],
            'weapon': ['banana'],
        }

    """
    groupid_to_items = defaultdict(list)
    for tags, item in zip(tags_list, item_list):
        for tag in tags:
            groupid_to_items[tag].append(item)
    return groupid_to_items


def groupby_attr(item_list, attrname):
    return group_items(item_list,
                       map(op.attrgetter(attrname), item_list))


def group_pairs(pair_list):
    """
    Groups a list of items using the first element in each pair as the item and
    the second element as the groupid.

    Args:
        pair_list (list): list of 2-tuples (item, groupid)

    Returns:
        dict: groupid_to_items: maps a groupid to a list of items

    SeeAlso:
        group_items
    """
    # Initialize dict of lists
    groupid_to_items = defaultdict(list)
    # Insert each item into the correct group
    for item, groupid in pair_list:
        groupid_to_items[groupid].append(item)
    return groupid_to_items


def group_items(items, by=None, sorted_=True):
    """
    Groups a list of items by group id.

    Args:
        items (list): a list of the values to be grouped.
            if `by` is None, then each item is assumed to be a
            (groupid, value) pair.
        by (list): a corresponding list to group items by.
            if specified, these are used as the keys to group values
            in `items`
        sorted_ (bool): if True preserves the ordering of items within groups
            (default = True) FIXME. the opposite is true

    Returns:
        dict: groupid_to_items: maps a groupid to a list of items

    SeeAlso:
        group_indices - first part of a a more fine grained grouping algorithm
        apply_gropuing - second part of a more fine grained grouping algorithm

    CommandLine:
        python -m utool.util_dict --test-group_items

    Example:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> items = ['ham',     'jam',   'spam',     'eggs',    'cheese', 'bannana']
        >>> by    = ['protein', 'fruit', 'protein',  'protein', 'dairy',  'fruit']
        >>> groupid_to_items = ut.group_items(items, iter(by))
        >>> result = ut.repr2(groupid_to_items)
        >>> print(result)
        {'dairy': ['cheese'], 'fruit': ['jam', 'bannana'], 'protein': ['ham', 'spam', 'eggs']}
    """
    if by is not None:
        pairs = list(zip(by, items))
        if sorted_:
            # Sort by groupid for cache efficiency (does this even do anything?)
            # I forgot why this is needed? Determenism?
            try:
                pairs = sorted(pairs, key=op.itemgetter(0))
            except TypeError:
                # Python 3 does not allow sorting mixed types
                pairs = sorted(pairs, key=lambda tup: str(tup[0]))
    else:
        pairs = items

    # Initialize a dict of lists
    groupid_to_items = defaultdict(list)
    # Insert each item into the correct group
    for groupid, item in pairs:
        groupid_to_items[groupid].append(item)
    return groupid_to_items


def hierarchical_group_items(item_list, groupids_list):
    """
    Generalization of group_item. Convert a flast list of ids into a heirarchical dictionary.

    TODO: move to util_dict

    Reference:
        http://stackoverflow.com/questions/10193235/python-translate-a-table-to-a-hierarchical-dictionary

    Args:
        item_list (list):
        groupids_list (list):

    CommandLine:
        python -m utool.util_dict --exec-hierarchical_group_items

    Example:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> item_list     = [1, 2, 3, 4]
        >>> groupids_list = [[1, 1, 2, 2]]
        >>> tree = hierarchical_group_items(item_list, groupids_list)
        >>> result = ('tree = ' + ut.repr4(tree, nl=len(groupids_list) - 1))
        >>> print(result)
        tree = {1: [1, 2], 2: [3, 4]}

    Example:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> item_list     = [1, 2, 3, 4, 5, 6, 7, 8]
        >>> groupids_list = [[1, 2, 1, 2, 1, 2, 1, 2], [3, 2, 2, 2, 3, 1, 1, 1]]
        >>> tree = hierarchical_group_items(item_list, groupids_list)
        >>> result = ('tree = ' + ut.repr4(tree, nl=len(groupids_list) - 1))
        >>> print(result)
        tree = {
            1: {1: [7], 2: [3], 3: [1, 5]},
            2: {1: [6, 8], 2: [2, 4]},
        }

    Example:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> item_list     = [1, 2, 3, 4]
        >>> groupids_list = [[1, 1, 1, 2], [1, 2, 2, 2], [1, 3, 1, 1]]
        >>> tree = hierarchical_group_items(item_list, groupids_list)
        >>> result = ('tree = ' + ut.repr4(tree, nl=len(groupids_list) - 1))
        >>> print(result)
        tree = {
            1: {
                1: {1: [1]},
                2: {1: [3], 3: [2]},
            },
            2: {
                2: {1: [4]},
            },
        }

    """
    # Construct a defaultdict type with the appropriate number of levels
    num_groups = len(groupids_list)
    leaf_type = partial(defaultdict, list)
    if num_groups > 1:
        node_type = leaf_type
        for _ in range(len(groupids_list) - 2):
            node_type = partial(defaultdict, node_type)
        root_type = node_type
    elif num_groups == 1:
        root_type = list
    else:
        raise ValueError('must suply groupids')
    tree = defaultdict(root_type)
    #
    groupid_tuple_list = list(zip(*groupids_list))
    for groupid_tuple, item in zip(groupid_tuple_list, item_list):
        node = tree
        for groupid in groupid_tuple:
            node = node[groupid]
        node.append(item)
    return tree


def iflatten_dict_values(node, depth=0):
    """
        >>> from utool.util_dict import *  # NOQA
    """
    if isinstance(node, dict):
        _iter = (iflatten_dict_values(value) for value in six.itervalues(node))
        return util_iter.iflatten(_iter)
    else:
        return node


#def iflatten_dict_items(node, depth=0):
#    if isinstance(node, dict):
#        six.iteritems(node)
#        _iter = ((key, iflatten_dict_items(value)) for key, value in six.iteritems(node))
#        return util_iter.iflatten(_iter)
#    else:
#        return node


#def iflatten_dict_keys(node, depth=0):
#    if isinstance(node, dict):
#        _iter = (iflatten_dict_keys(value) for key, value in six.iteritems(node))
#        return util_iter.iflatten(_iter)
#    else:
#        return node


def hierarchical_map_vals(func, node, max_depth=None, depth=0):
    """
    node is a dict tree like structure with leaves of type list

    TODO: move to util_dict

    CommandLine:
        python -m utool.util_dict --exec-hierarchical_map_vals

    Example:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> item_list     = [1, 2, 3, 4, 5, 6, 7, 8]
        >>> groupids_list = [[1, 2, 1, 2, 1, 2, 1, 2], [3, 2, 2, 2, 3, 1, 1, 1]]
        >>> tree = ut.hierarchical_group_items(item_list, groupids_list)
        >>> len_tree = ut.hierarchical_map_vals(len, tree)
        >>> result = ('len_tree = ' + ut.repr4(len_tree, nl=1))
        >>> print(result)
        len_tree = {
            1: {1: 1, 2: 1, 3: 2},
            2: {1: 2, 2: 2},
        }

    Example1:
        >>> # DISABLE_DOCTEST
        >>> # UNSTABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> depth = 4
        >>> item_list = list(range(2 ** (depth + 1)))
        >>> num = len(item_list) // 2
        >>> groupids_list = []
        >>> total = 0
        >>> for level in range(depth):
        ...     num2 = len(item_list) // int((num * 2))
        ...     #nonflat_levelids = [([total + 2 * x + 1] * num + [total + 2 * x + 2] * num) for x in range(num2)]
        ...     nonflat_levelids = [([1] * num + [2] * num) for x in range(num2)]
        ...     levelids = ut.flatten(nonflat_levelids)
        ...     groupids_list.append(levelids)
        ...     total += num2 * 2
        ...     num //= 2
        >>> print('groupids_list = %s' % (ut.repr4(groupids_list, nl=1),))
        >>> print('depth = %r' % (len(groupids_list),))
        >>> tree = ut.hierarchical_group_items(item_list, groupids_list)
        >>> print('tree = ' + ut.repr4(tree, nl=None))
        >>> flat_tree_values = list(ut.iflatten_dict_values(tree))
        >>> assert sorted(flat_tree_values) == sorted(item_list)
        >>> print('flat_tree_values = ' + str(flat_tree_values))
        >>> #print('flat_tree_keys = ' + str(list(ut.iflatten_dict_keys(tree))))
        >>> #print('iflatten_dict_items = ' + str(list(ut.iflatten_dict_items(tree))))
        >>> len_tree = ut.hierarchical_map_vals(len, tree, max_depth=4)
        >>> result = ('len_tree = ' + ut.repr4(len_tree, nl=None))
        >>> print(result)

    """
    #if not isinstance(node, dict):
    if not hasattr(node, 'items'):
        return func(node)
    elif max_depth is not None and depth >= max_depth:
        #return func(node)
        return map_dict_vals(func, node)
        #return {key: func(val) for key, val in six.iteritems(node)}
    else:
        # recursion
        #return {key: hierarchical_map_vals(func, val, max_depth, depth + 1) for key, val in six.iteritems(node)}
        #keyval_list = [(key, hierarchical_map_vals(func, val, max_depth, depth + 1)) for key, val in six.iteritems(node)]
        keyval_list = [(key, hierarchical_map_vals(func, val, max_depth, depth + 1)) for key, val in node.items()]
        if isinstance(node, OrderedDict):
            return OrderedDict(keyval_list)
        else:
            return dict(keyval_list)


def move_odict_item(odict, key, newpos):
    """
    References:
        http://stackoverflow.com/questions/22663966/changing-order-of-ordered-dictionary-in-python

    CommandLine:
        python -m utool.util_dict --exec-move_odict_item

    Example:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> odict = OrderedDict()
        >>> odict['a'] = 1
        >>> odict['b'] = 2
        >>> odict['c'] = 3
        >>> odict['e'] = 5
        >>> print(ut.repr4(odict, nl=False))
        >>> move_odict_item(odict, 'c', 1)
        >>> print(ut.repr4(odict, nl=False))
        >>> move_odict_item(odict, 'a', 3)
        >>> print(ut.repr4(odict, nl=False))
        >>> move_odict_item(odict, 'a', 0)
        >>> print(ut.repr4(odict, nl=False))
        >>> move_odict_item(odict, 'b', 2)
        >>> result = ut.repr4(odict, nl=False)
        >>> print(result)
        {'a': 1, 'c': 3, 'b': 2, 'e': 5}
    """
    odict[key] = odict.pop(key)
    for i, otherkey in enumerate(list(odict.keys())):
        if otherkey != key and i >= newpos:
            odict[otherkey] = odict.pop(otherkey)
    return odict


hmap_vals = hierarchical_map_vals


#def hierarchical_map_nodes(func, node, max_depth=None, depth=0):
#    """
#    applies function to non-leaf nodes
#    """
#    if not isinstance(node, dict):
#        return node
#    elif max_depth is not None and depth >= max_depth:
#        #return func(node)
#        return func(node)
#    else:
#        # recursion
#        return {key: func(hierarchical_map_vals(func, val, max_depth, depth + 1)) for key, val in six.iteritems(node)}


class DictLike(object):
    """
    move to util_dict rectify with util_dev

    An inherited class must specify the ``getitem``, ``setitem``, and
      ``keys`` methods.


    """

    def getitem(self, key):
        raise NotImplementedError('abstract getitem function')

    def setitem(self, key, value):
        raise NotImplementedError('abstract setitem function')

    def delitem(self, key):
        raise NotImplementedError('abstract delitem function')

    def keys(self):
        raise NotImplementedError('abstract keys function')

    def __repr__(self):
        return repr(self.asdict())

    def __str__(self):
        return str(self.asdict())

    def __len__(self):
        return len(list(self.keys()))

    def __contains__(self, key):
        return key in self.keys()

    def __delitem__(self, key):
        return self.delitem(key)

    def __getitem__(self, key):
        return self.getitem(key)

    def __setitem__(self, key, value):
        return self.setitem(key, value)

    def items(self):
        if six.PY2:
            return list(self.iteritems())
        else:
            return self.iteritems()

    def values(self):
        if six.PY2:
            return [self[key] for key in self.keys()]
        else:
            return (self[key] for key in self.keys())

    def copy(self):
        return dict(self.items())

    def asdict(self):
        return dict(self.items())

    def iteritems(self):
        for key, val in zip(self.iterkeys(), self.itervalues()):
            yield key, val

    def itervalues(self):
        return (self[key] for key in self.keys())

    def iterkeys(self):
        return (key for key in self.keys())

    def get(self, key, default=None):
        try:
            return self[key]
        except KeyError:
            return default


def sort_dict(dict_, part='keys', key=None, reverse=False):
    """
    sorts a dictionary by its values or its keys

    Args:
        dict_ (dict_):  a dictionary
        part (str): specifies to sort by keys or values
        key (Optional[func]): a function that takes specified part
            and returns a sortable value
        reverse (bool): (Defaults to False) - True for descinding order. False
            for ascending order.

    Returns:
        OrderedDict: sorted dictionary

    CommandLine:
        python -m utool.util_dict sort_dict

    Example:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> dict_ = {'a': 3, 'c': 2, 'b': 1}
        >>> results = []
        >>> results.append(sort_dict(dict_, 'keys'))
        >>> results.append(sort_dict(dict_, 'vals'))
        >>> results.append(sort_dict(dict_, 'vals', lambda x: -x))
        >>> result = ut.repr4(results)
        >>> print(result)
        [
            {'a': 3, 'b': 1, 'c': 2},
            {'b': 1, 'c': 2, 'a': 3},
            {'a': 3, 'c': 2, 'b': 1},
        ]
    """
    if part == 'keys':
        index = 0
    elif part in {'vals', 'values'}:
        index = 1
    else:
        raise ValueError('Unknown method part=%r' % (part,))
    if key is None:
        _key = op.itemgetter(index)
    else:
        def _key(item):
            return key(item[index])
    sorted_items = sorted(six.iteritems(dict_), key=_key, reverse=reverse)
    sorted_dict = OrderedDict(sorted_items)
    return sorted_dict


def order_dict_by(dict_, key_order):
    r"""
    Reorders items in a dictionary according to a custom key order

    Args:
        dict_ (dict_):  a dictionary
        key_order (list): custom key order

    Returns:
        OrderedDict: sorted_dict

    CommandLine:
        python -m utool.util_dict --exec-order_dict_by

    Example:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> dict_ = {1: 1, 2: 2, 3: 3, 4: 4}
        >>> key_order = [4, 2, 3, 1]
        >>> sorted_dict = order_dict_by(dict_, key_order)
        >>> result = ('sorted_dict = %s' % (ut.repr4(sorted_dict, nl=False),))
        >>> print(result)
        >>> assert result == 'sorted_dict = {4: 4, 2: 2, 3: 3, 1: 1}'

    """
    dict_keys = set(dict_.keys())
    other_keys = dict_keys - set(key_order)
    key_order = it.chain(key_order, other_keys)
    sorted_dict = OrderedDict(
        (key, dict_[key]) for key in key_order if key in dict_keys
    )
    return sorted_dict


def iteritems_sorted(dict_):
    """ change to iteritems ordered """
    if isinstance(dict_, OrderedDict):
        return six.iteritems(dict_)
    else:
        return iter(sorted(six.iteritems(dict_)))


def keys_sorted_by_value(dict_):
    sorted_keys = sorted(dict_, key=lambda key: dict_[key])
    return sorted_keys


def flatten_dict_vals(dict_):
    """
    Flattens only values in a heirarchical dictionary, keys are nested.
    """
    if isinstance(dict_, dict):
        return dict([
            ((key, augkey), augval)
            for key, val in dict_.items()
            for augkey, augval in flatten_dict_vals(val).items()
        ])
    else:
        return {None: dict_}


def flatten_dict_items(dict_):
    """
    Flattens keys / values in a heirarchical dictionary

    Example:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> item_list     = [1, 2, 3, 4]
        >>> groupids_list = [[1, 1, 1, 2], [1, 2, 2, 2], [1, 3, 1, 1]]
        >>> dict_ = hierarchical_group_items(item_list, groupids_list)
        >>> flatter_dict = flatten_dict_items(dict_)
        >>> result = ('flatter_dict = ' + ut.repr4(flatter_dict, nl=1))
        >>> print(result)
        flatter_dict = {
            (1, 1, 1): [1],
            (1, 2, 1): [3],
            (1, 2, 3): [2],
            (2, 2, 1): [4],
        }
    """
    import utool as ut
    flat_dict = ut.flatten_dict_vals(dict_)
    flatter_dict = dict([(tuple(ut.unpack_iterables(key)[:-1]), val)
                         for key, val in flat_dict.items()])
    return flatter_dict


def depth_atleast(list_, depth):
    r"""
    Returns if depth of list is at least ``depth``

    Args:
        list_ (list):
        depth (int):

    Returns:
        bool: True

    CommandLine:
        python -m utool.util_dict --exec-depth_atleast --show

    Example:
        >>> # DISABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> list_ = [[[[0]]], [[0]]]
        >>> depth = 0
        >>> result = [depth_atleast(list_, depth) for depth in range(0, 7)]
        >>> print(result)
    """
    if depth == 0:
        return True
    else:
        try:
            return all([depth_atleast(item, depth - 1) for item in list_])
        except TypeError:
            return False


class DefaultValueDict(dict):
    """
    picklable default dictionary that can store scalar values.

    Example:
        >>> # DISABLE_DOCTEST
        >>> from utool.util_dict import *  # NOQA
        >>> import utool as ut
        >>> self = ut.DefaultValueDict(0)
        >>> print(self[4])
        >>> self[4] = 4
        >>> print(self[4])
        0
        4
    """
    def __init__(self, default, other=None, **kwargs):
        self.default = default
        if other:
            self.update(other)
        if kwargs:
            self.update(kwargs)

    def __getitem__(self, item):
        return self.get(item, self.default)


if __name__ == '__main__':
    """
    CommandLine:
        python -m utool.util_dict
        python -m utool.util_dict --allexamples
    """
    import multiprocessing
    multiprocessing.freeze_support()  # for win32
    import utool as ut  # NOQA
    ut.doctest_funcs()



# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, print_function
try:
    import numpy as np
    HAVE_NUMPY = True
except ImportError:
    HAVE_NUMPY = False
    # TODO remove numpy
    pass
import operator
from six.moves import zip
from utool import util_iter
from utool import util_alg
from utool import util_inject
print, rrr, profile = util_inject.inject2(__name__)
from utool import util_arg  # NOQA


def get_first_None_position(list_):
    for index, item in enumerate(list_):
        if item is None:
            return index
    return None


def assert_raises(ex_type, func, *args, **kwargs):
    r"""
    Checks that a function raises an error when given specific arguments.

    Args:
        ex_type (Exception): exception type
        func (callable): live python function

    CommandLine:
        python -m utool.util_assert assert_raises --show

    Example:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_assert import *  # NOQA
        >>> import utool as ut
        >>> ex_type = AssertionError
        >>> func = len
        >>> # Check that this raises an error when something else does not
        >>> assert_raises(ex_type, assert_raises, ex_type, func, [])
        >>> # Check this does not raise an error when something else does
        >>> assert_raises(ValueError, [].index, 0)
    """
    try:
        func(*args, **kwargs)
    except Exception as ex:
        assert isinstance(ex, ex_type), (
            'Raised %r but type should have been %r' % (ex, ex_type))
        return True
    else:
        raise AssertionError('No error was raised')


def assert_unique(item_list, ignore=[], name='list', verbose=None):
    import utool as ut
    dups = ut.find_duplicate_items(item_list)
    ut.delete_dict_keys(dups, ignore)
    if len(dups) > 0:
        raise AssertionError(
            'Found duplicate items in %s: %s' % (
                name, ut.repr4(dups)))
    if verbose:
        print('No duplicates found in %s' % (name,))


def assert_all_in(key_list, valid_list, msg=''):
    missing_keys = set(key_list).difference(set(valid_list))
    assert len(missing_keys) == 0, 'missing_keys = %r. %s' % (missing_keys, msg)


def assert_all_not_None(list_, list_name='some_list', key_list=[], verbose=not
                        util_arg.QUIET, veryverbose=False):
    if util_arg.NO_ASSERTS:
        return
    try:
        index = get_first_None_position(list_)
        assert index is None, 'index=%r in %s is None' % (index, list_name)
        if veryverbose:
            print('PASSED: %s has no Nones' % (list_name))
    except AssertionError as ex:
        from utool import util_dbg
        item = list_[index]
        msg = (list_name + '[%d] = %r') % (index, item)
        if verbose:
            msg += '\n len(list_) = %r' % (len(list_))
        util_dbg.printex(ex, msg, keys=key_list, N=1)
        raise


def assert_unflat_level(unflat_list, level=1, basetype=None):
    if util_arg.NO_ASSERTS:
        return
    num_checked = 0
    for item in unflat_list:
        if level == 1:
            for x in item:
                num_checked += 1
                assert not isinstance(x, (tuple, list)), \
                    'list is at an unexpected unflat level, x=%r' % (x,)
                if basetype is not None:
                    assert isinstance(x, basetype), \
                        'x=%r, type(x)=%r is not basetype=%r' % (x, type(x), basetype)
        else:
            assert_unflat_level(item, level - 1)
    #print('checked %r' % num_checked)
    #assert num_checked > 0, 'num_checked=%r' % num_checked


def assert_scalar_list(list_):
    if util_arg.NO_ASSERTS:
        return
    for count, item in enumerate(list_):
        assert not util_iter.isiterable(item), 'count=%r, item=%r is iterable!' % (count, item)


def assert_same_len(list1, list2, additional_msg=''):
    if util_arg.NO_ASSERTS:
        return
    assert len(list1) == len(list2), (
        'unequal lens. len(list1)=%r, len(list2)=%r%s' % (
            len(list1), len(list2), additional_msg))


assert_eq_len = assert_same_len


def lists_eq(list1, list2):
    """ recursive """
    if len(list1) != len(list2):
        return False
    for count, (item1, item2) in enumerate(zip(list1, list2)):
        if isinstance(item1, np.ndarray) or isinstance(item2, np.ndarray):
            failed = not np.all(item1 == item2)  # lists_eq(item1, item2)
        else:
            failed = item1 != item2
        if failed:
            return False
    return True


def assert_lists_eq(list1, list2, failmsg='', verbose=False):
    if util_arg.NO_ASSERTS:
        return
    msg = ''
    if len(list1) != len(list2):
        msg += ('LENGTHS ARE UNEQUAL: len(list1)=%r, len(list2)=%r\n' % (len(list1), len(list2)))

    difflist = []
    for count, (item1, item2) in enumerate(zip(list1, list2)):
        if item1 != item2:
            difflist.append('count=%r, item1=%r, item2=%r' % (count, item1, item2))

    length = max(len(list1), len(list2))

    if verbose or len(difflist) < 10:
        msg += '\n'.join(difflist)
    else:
        if len(difflist) > 0:
            msg += 'There are %d/%d different ordered items\n' % (len(difflist), length)

    if len(msg) > 0:
        intersecting_items = set(list1).intersection(set(list2))
        missing_items1 = set(list2).difference(intersecting_items)
        missing_items2 = set(list1).difference(intersecting_items)
        num_intersect = len(intersecting_items)
        isect_msg = 'There are %d/%d intersecting unordered items' % (num_intersect, length)
        msg = failmsg + '\n' + msg + isect_msg
        if len(missing_items1) > 0:
            msg += '\n %d items are missing from list1' % (len(missing_items1))
            msg += '\n missing_items1 = %r' % (missing_items1,)
        if len(missing_items2) > 0:
            msg += '\n %d items are missing from list2' % (len(missing_items2))
            msg += '\n missing_items2 = %r' % (missing_items2,)
        ex = AssertionError(msg)
        if verbose:
            print(msg)
        raise ex


def assert_inbounds(num, low, high, msg='', eq=False, verbose=not util_arg.QUIET):
    r"""
    Args:
        num (scalar):
        low (scalar):
        high (scalar):
        msg (str):
    """
    from utool import util_str
    if util_arg.NO_ASSERTS:
        return
    passed = util_alg.inbounds(num, low, high, eq=eq)
    if isinstance(passed, np.ndarray):
        passflag = np.all(passed)
    else:
        passflag = passed
    if not passflag:
        failednum = num.compress(~passed) if isinstance(num, np.ndarray) else num
        failedlow = low.compress(~passed) if isinstance(low, np.ndarray) else low
        failedhigh = high.compress(~passed) if isinstance(high, np.ndarray) else high
        msg_ = 'num=%r is out of bounds=(%r, %r)' % (failednum, failedlow, failedhigh)
        raise AssertionError(msg_ + '\n' + msg)
    else:
        if verbose:
            op = '<=' if eq else '<'
            fmtstr = 'Passed assert_inbounds: {low} {op} {num} {op} {high}'
            print(fmtstr.format(low=low, op=op, num=util_str.truncate_str(str(num)), high=high))


def assert_almost_eq(arr_test, arr_target, thresh=1E-11):
    r"""
    Args:
        arr_test (ndarray or list):
        arr_target (ndarray or list):
        thresh (scalar or ndarray or list):
    """
    if util_arg.NO_ASSERTS:
        return
    import utool as ut
    arr1 = np.array(arr_test)
    arr2 = np.array(arr_target)
    passed, error = ut.almost_eq(arr1, arr2, thresh, ret_error=True)
    if not np.all(passed):
        failed_xs = np.where(np.logical_not(passed))
        failed_error = error.take(failed_xs)
        failed_arr_test = arr1.take(failed_xs)
        failed_arr_target = arr2.take(failed_xs)

        msg_list = [
            'FAILED ASSERT ALMOST EQUAL',
            '  * failed_xs = %r' % (failed_xs,),
            '  * failed_error = %r' % (failed_error,),
            '  * failed_arr_test   = %r' % (failed_arr_test,),
            '  * failed_arr_target = %r' % (failed_arr_target,),
        ]
        msg = '\n'.join(msg_list)
        raise AssertionError(msg)
    return error


def assert_lessthan(arr_test, arr_max, msg=''):
    r"""
    Args:
        arr_test (ndarray or list):
        arr_target (ndarray or list):
        thresh (scalar or ndarray or list):
    """
    if util_arg.NO_ASSERTS:
        return
    arr1 = np.array(arr_test)
    arr2 = np.array(arr_max)
    error = arr_max - arr_test
    passed = error >= 0
    if not np.all(passed):
        failed_xs = np.where(np.logical_not(passed))
        failed_error = error.take(failed_xs)
        failed_arr_test = arr1.take(failed_xs)
        failed_arr_target = arr2.take(failed_xs)

        msg_list = [
            'FAILED ASSERT LESSTHAN',
            msg,
            '  * failed_xs = %r' % (failed_xs,),
            '  * failed_error = %r' % (failed_error,),
            '  * failed_arr_test   = %r' % (failed_arr_test,),
            '  * failed_arr_target = %r' % (failed_arr_target,),
        ]
        msg = '\n'.join(msg_list)
        raise AssertionError(msg)
    return error


def assert_all_eq(item_list, eq_=operator.eq):
    if len(item_list) == 0:
        return True
    import six
    item_iter = iter(item_list)
    item0 = six.next(item_iter)
    for count, item in enumerate(item_iter, start=1):
        flag = eq_(item0, item)
        if not flag:
            print('Error:')
            print('count = %r' % (count,))
            print('item = %r' % (item,))
            print('item0 = %r' % (item0,))
            msg = 'The %d-th item was not equal to item 0' % (count,)
            raise AssertionError(msg)


def assert_eq(var1, var2, msg='', var1_name=None, var2_name=None,
              verbose=None):
    import utool as ut
    if verbose is None:
        verbose = not util_arg.QUIET
    failed = var1 != var2
    if var1_name is None:
        var1_name = ut.get_varname_from_stack(var1, N=1, default='var1')
    if var2_name is None:
        var2_name = ut.get_varname_from_stack(var2, N=1, default='var2')
    fmtdict = dict(
        msg=msg,
        var1_name=var1_name,
        var2_name=var2_name,
        var1_repr=repr(var1),
        var2_repr=repr(var2))
    if failed:
        msg_fmtstr = ut.codeblock('''
            +=====
            ERROR {var1_name} != {var2_name}
            msg = {msg}
            ---
            {var1_name} = {var1_repr}
            ---
            {var2_name} = {var2_repr}
            L_____
            ''')
        msg = msg_fmtstr.format(**fmtdict)
        raise AssertionError(msg)
    else:
        if verbose:
            print('ASSERT_EQ_PASSED: {var1_name} == {var2_name} == {var1_repr}'.format(**fmtdict))


if __name__ == '__main__':
    """
    CommandLine:
        python -m utool.util_assert
        python -m utool.util_assert --allexamples
        python -m utool.util_assert --allexamples --noface --nosrc
    """
    import multiprocessing
    multiprocessing.freeze_support()  # for win32
    import utool as ut  # NOQA
    ut.doctest_funcs()



# -*- coding: utf-8 -*-
"""
Handles command line parsing
"""
from __future__ import absolute_import, division, print_function, unicode_literals
import sys
import six
import os
from six.moves import builtins
import itertools
import argparse
from utool import util_inject
from utool import util_type
from utool._internal import meta_util_six, meta_util_arg, meta_util_iter
print, rrr, profile = util_inject.inject2(__name__)

#STRICT = '--nostrict' not in sys.argv
DEBUG2       = meta_util_arg.DEBUG2
NO_ASSERTS   = meta_util_arg.NO_ASSERTS
SAFE         = meta_util_arg.SAFE
STRICT       = meta_util_arg.STRICT
REPORT       = meta_util_arg.REPORT
SUPER_STRICT = meta_util_arg.SUPER_STRICT
TRACE        = meta_util_arg.TRACE
USE_ASSERT   = meta_util_arg.USE_ASSERT
SILENT       = meta_util_arg.SILENT
VERBOSE      = meta_util_arg.VERBOSE
VERYVERBOSE  = meta_util_arg.VERYVERBOSE
NOT_QUIET    = meta_util_arg.NOT_QUIET
QUIET        = meta_util_arg.QUIET

DEBUG = 0


#(switch, type, default, help)
# TODO: make a static help file available via printing
__REGISTERED_ARGS__ = []


def aug_sysargv(cmdstr):
    """ DEBUG FUNC modify argv to look like you ran a command """
    import shlex
    argv = shlex.split(cmdstr)
    sys.argv.extend(argv)
    #return argv


def get_module_verbosity_flags(*labels):
    """ checks for standard flags for enableing module specific verbosity """
    verbose_prefix_list = ['--verbose-', '--verb', '--verb-']
    veryverbose_prefix_list = ['--veryverbose-', '--veryverb', '--veryverb-']
    verbose_flags = tuple(
        [prefix + lbl for prefix, lbl in
         itertools.product(verbose_prefix_list, labels)])
    veryverbose_flags = tuple(
        [prefix + lbl for prefix, lbl in
         itertools.product(veryverbose_prefix_list, labels)])
    veryverbose_module = get_argflag(veryverbose_flags) or VERYVERBOSE
    verbose_module = (get_argflag(verbose_flags) or veryverbose_module or VERBOSE)
    if veryverbose_module:
        verbose_module = 2
    return verbose_module, veryverbose_module

get_verbflag = get_module_verbosity_flags


def reset_argrecord():
    """ forgets about the args already parsed """
    global __REGISTERED_ARGS__
    __REGISTERED_ARGS__ = []


def _register_arg(argstr_list, type_, default, help_):
    # TODO REGISTER PARENTS
    global __REGISTERED_ARGS__
    __REGISTERED_ARGS__.append((argstr_list, type_, default, help_))


def autogen_argparse_block(extra_args=[]):
    """
    SHOULD TURN ANY REGISTERED ARGS INTO A A NEW PARSING CONFIG
    FILE FOR BETTER --help COMMANDS

    import utool as ut
    __REGISTERED_ARGS__ = ut.util_arg.__REGISTERED_ARGS__

    Args:
        extra_args (list): (default = [])

    CommandLine:
        python -m utool.util_arg --test-autogen_argparse_block

    Example:
        >>> # DISABLE_DOCTEST
        >>> import utool as ut
        >>> extra_args = []
        >>> result = ut.autogen_argparse_block(extra_args)
        >>> print(result)
    """
    #import utool as ut  # NOQA
    #__REGISTERED_ARGS__
    # TODO FINISHME

    grouped_args = []
    # Group similar a args
    for argtup in __REGISTERED_ARGS__:
        argstr_list, type_, default, help_ = argtup
        argstr_set = set(argstr_list)
        # <MULTIKEY_SETATTR>
        # hack in multikey setattr n**2 yuck
        found = False
        for index, (keyset, vals) in enumerate(grouped_args):
            if len(keyset.intersection(argstr_set)) > 0:
                # update
                keyset.update(argstr_set)
                vals.append(argtup)
                found = True
                break
        if not found:
            new_keyset = argstr_set
            new_vals = [argtup]
            grouped_args.append((new_keyset, new_vals))
        # </MULTIKEY_SETATTR>
    # DEBUG
    multi_groups = []
    for keyset, vals in grouped_args:
        if len(vals) > 1:
            multi_groups.append(vals)
    if len(multi_groups) > 0:
        import utool as ut
        print('Following arg was specified multiple times')
        print(ut.repr4(multi_groups, newlines=2))


def get_argflag(argstr_, default=False, help_='', return_specified=None,
                need_prefix=True, return_was_specified=False, argv=None,
                debug=None,
                **kwargs):
    """
    Checks if the commandline has a flag or a corresponding noflag

    Args:
        argstr_ (str, list, or tuple): the flag to look for
        default (bool): dont use this (default = False)
        help_ (str): a help string (default = '')
        return_specified (bool): returns if flag was specified or not (default = False)

    Returns:
        tuple: (parsed_val, was_specified)

    TODO:
        depricate return_was_specified

    CommandLine:
        python -m utool.util_arg --exec-get_argflag --noface --exec-mode
        python -m utool.util_arg --exec-get_argflag --foo --exec-mode
        python -m utool.util_arg --exec-get_argflag --no-foo --exec-mode
        python -m utool.util_arg --exec-get_argflag --foo=True --exec-mode
        python -m utool.util_arg --exec-get_argflag --foo=False  --exec-mode

    Example:
        >>> # DISABLE_DOCTEST
        >>> from utool.util_arg import *  # NOQA
        >>> argstr_ = '--foo'
        >>> default = False
        >>> help_ = ''
        >>> return_specified = True
        >>> (parsed_val, was_specified) = get_argflag(argstr_, default, help_, return_specified)
        >>> result = ('(parsed_val, was_specified) = %s' % (str((parsed_val, was_specified)),))
        >>> print(result)
    """
    if argv is None:
        argv = sys.argv
    assert isinstance(default, bool), 'default must be boolean'
    argstr_list = meta_util_iter.ensure_iterable(argstr_)
    #if VERYVERBOSE:
    #    print('[util_arg] checking argstr_list=%r' % (argstr_list,))
    # arg registration
    _register_arg(argstr_list, bool, default, help_)
    parsed_val = default
    was_specified = False

    if debug is None:
        debug = DEBUG

    # Check environment variables for default as well as argv
    import os
    #"""
    #set UTOOL_NOCNN=True
    #export UTOOL_NOCNN True
    #"""
    #argv_orig = argv[:]
    # HACK: make this not happen very time you loop
    for key, val in os.environ.items():
        key = key.upper()
        sentinal = 'UTOOL_'
        if key.startswith(sentinal):
            flag = '--' + key[len(sentinal):].lower().replace('_', '-')
            if val.upper() in ['TRUE', 'ON']:
                pass
            elif val.upper() in ['FALSE', 'OFF']:
                continue
            else:
                continue
                #flag += '=False'
            new_argv = [flag]
            argv = argv[:] + new_argv
            if debug:
                print('ENV SPECIFIED COMMAND LINE')
                print('argv.extend(new_argv=%r)' % (new_argv,))

    for argstr in argstr_list:
        #if VERYVERBOSE:
        #    print('[util_arg]   * checking argstr=%r' % (argstr,))
        if not (argstr.find('--') == 0 or (argstr.find('-') == 0 and len(argstr) == 2)):
            raise AssertionError('Invalid argstr: %r' % (argstr,))
        if not need_prefix:
            noprefix = argstr.replace('--', '')
            if noprefix in argv:
                parsed_val = True
                was_specified = True
                break
        #if argstr.find('--no') == 0:
            #argstr = argstr.replace('--no', '--')
        noarg = argstr.replace('--', '--no')
        if argstr in argv:
            parsed_val = True
            was_specified = True
            #if VERYVERBOSE:
            #    print('[util_arg]   * ...WAS_SPECIFIED. AND PARSED')
            break
        elif noarg in argv:
            parsed_val = False
            was_specified = True
            #if VERYVERBOSE:
            #    print('[util_arg]   * ...WAS_SPECIFIED. AND NOT PARSED')
            break
        elif argstr + '=True' in argv:
            parsed_val = True
            was_specified = True
            break
        elif argstr + '=False' in argv:
            parsed_val = False
            was_specified = True
            break

    if return_specified is None:
        return_specified = return_was_specified

    if return_specified:
        return parsed_val, was_specified
    else:
        return parsed_val


# TODO: rectify with meta_util_arg
# This has diverged and is now better
#from utool._internal.meta_util_arg import get_argval
#@profile
def get_argval(argstr_, type_=None, default=None, help_=None, smartcast=True,
               return_specified=None, argv=None, verbose=None,
               debug=None, return_was_specified=False, pos=None):
    r"""
    Returns a value of an argument specified on the command line after some flag

    Args:
        argstr_ (str or tuple): string or tuple of strings denoting the command line values to parse
        type_ (None): type of the variable to parse (default = None)
        default (None): (default = None)
        help_ (None): help for this argument (not fully integrated) (default = None)
        smartcast (bool): tries to be smart about casting the parsed strings (default = True)
        return_specified (bool): (default = False)
        argv (None): override sys.argv with custom command line vector (default = None)
        pos (int): if specified the argument can also be found in position `pos` of the command line varargs

    TODO:
        depricate return_was_specified

    CommandLine:
        python -m utool.util_arg --test-get_argval
        python -m utool.util_arg --exec-get_argval:0
        python -m utool.util_arg --exec-get_argval:1
        python -c "import utool; print([(type(x), x) for x in [utool.get_argval('--quest')]])" --quest="holy grail"
        python -c "import utool; print([(type(x), x) for x in [utool.get_argval('--quest')]])" --quest="42"
        python -c "import utool; print([(type(x), x) for x in [utool.get_argval('--quest')]])" --quest=42
        python -c "import utool; print([(type(x), x) for x in [utool.get_argval('--quest')]])" --quest 42
        python -c "import utool; print([(type(x), x) for x in [utool.get_argval('--quest', float)]])" --quest 42
        python -c "import utool; print([(type(x), x) for x in [utool.get_argval(('--nAssign'), int)]])" --nAssign 42
        python -c "import utool; print([(type(x), x) for x in [utool.get_argval(('--test'), str)]])" --test
        python -c "import utool; print([(type(x), x) for x in [utool.get_argval(('--test'), str)]])" --test "foobar is good" --youbar ok

    Example:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_arg import *  # NOQA
        >>> import utool as ut
        >>> import sys
        >>> argv = ['--spam', 'eggs', '--quest=holy grail', '--ans=42', '--the-val=1,2,3']
        >>> # specify a list of args and kwargs to get_argval
        >>> argstr_kwargs_list = [
        >>>     ('--spam',                    dict(type_=str, default=None, argv=argv)),
        >>>     ('--quest',                   dict(type_=str, default=None, argv=argv)),
        >>>     (('--ans', '--foo'),          dict(type_=int, default=None, argv=argv)),
        >>>     (('--not-there', '--absent'), dict(argv=argv)),
        >>>     ('--the_val',                 dict(type_=list, argv=argv)),
        >>>     ('--the-val',                 dict(type_=list, argv=argv)),
        >>> ]
        >>> # Execute the command with for each of the test cases
        >>> res_list = []
        >>> argstr_list = ut.get_list_column(argstr_kwargs_list, 0)
        >>> for argstr_, kwargs in argstr_kwargs_list:
        >>>     res = get_argval(argstr_, **kwargs)
        >>>     res_list.append(res)
        >>> result = ut.repr2(ut.odict(zip(argstr_list, res_list)), nl=1)
        >>> result = result.replace('u\'', '\'')  # hack
        >>> print(result)
        {
            '--spam': 'eggs',
            '--quest': 'holy grail',
            ('--ans', '--foo'): 42,
            ('--not-there', '--absent'): None,
            '--the_val': [1, 2, 3],
            '--the-val': [1, 2, 3],
        }

    Example:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_arg import *  # NOQA
        >>> import utool as ut
        >>> import sys
        >>> argv = ['--slice1', '::', '--slice2=4:', '--slice3=::4', '--slice4', '[1,2,3,4]', '--slice5=3']
        >>> # specify a list of args and kwargs to get_argval
        >>> argstr_kwargs_list = [
        >>>     ('--slice1',            dict(type_='fuzzy_subset', default=None, argv=argv)),
        >>>     ('--slice2',            dict(type_='fuzzy_subset', default=None, argv=argv)),
        >>>     ('--slice3',            dict(type_='fuzzy_subset', default=None, argv=argv)),
        >>>     ('--slice4',            dict(type_='fuzzy_subset', default=None, argv=argv)),
        >>>     ('--slice5',            dict(type_='fuzzy_subset', default=None, argv=argv)),
        >>> ]
        >>> # Execute the command with for each of the test cases
        >>> res_list = []
        >>> argstr_list = ut.get_list_column(argstr_kwargs_list, 0)
        >>> list1 = [1, 3, 5, 7, 9]
        >>> import numpy as np
        >>> list2 = np.array([[1, 2], [3, 4], [5, 6], [7, 8], [9, 1]])
        >>> for argstr_, kwargs in argstr_kwargs_list:
        >>>     res = get_argval(argstr_, **kwargs)
        >>>     print('---')
        >>>     print('res = %r' % (res,))
        >>>     print('list1[%r=%r] = %r' % (argstr_, res, ut.take(list1, res),))
        >>>     print('list2[%r=%r] = %r' % (argstr_, res, list2[res].tolist(),))
        >>>     res_list.append(res)
        >>> result = ut.repr4(ut.odict(zip(argstr_list, res_list)))
        >>> result = result.replace('u\'', '\'')  # hack
        >>> print(result)

    """
    if verbose is None:
        pass
        # verbose = VERYVERBOSE

    if debug is None:
        debug = DEBUG
        # debug = VERYVERBOSE

    if argv is None:
        argv = sys.argv

    #verbose = 1

    if verbose:
        print('[get_argval] Searching Commandline for argstr_=%r' % (argstr_,))
        #print('[get_argval]  * type_ = %r' % (type_,))
        #print('[get_argval]  * default = %r' % (default,))
        #print('[get_argval]  * help_ = %r' % (help_,))
        #print('[get_argval]  * smartcast = %r' % (smartcast,))

    if return_specified is None:
        return_specified = return_was_specified

    #print(argstr_)
    was_specified = False
    arg_after = default
    if type_ is bool:
        arg_after = False if default is None else default
    try:
        # New for loop way (accounts for =)
        argstr_list = meta_util_iter.ensure_iterable(argstr_)
        # arg registration
        _register_arg(argstr_list, type_, default, help_)

        # expand out hypens
        EXPAND_HYPENS = True
        if EXPAND_HYPENS:
            argstr_list2 = []
            seen_ = set([])
            for argstr in argstr_list:
                if argstr not in seen_:
                    argstr_list2.append(argstr)
                    seen_.add(argstr)
                if argstr.startswith('--'):
                    num = 2
                elif argstr.startswith('-'):
                    num = 1
                else:
                    continue
                argstr2_0 = argstr[0:num] + argstr[num:].replace('_', '-')
                argstr2_1 = argstr[0:num] + argstr[num:].replace('-', '_')
                if argstr2_0 not  in seen_:
                    argstr_list2.append(argstr2_0)
                    seen_.add(argstr2_0)
                if argstr2_1 not  in seen_:
                    argstr_list2.append(argstr2_1)
                    seen_.add(argstr2_1)
            argstr_list = argstr_list2

        # Check environment variables for default as well as argv
        import os
        """
        set UTOOL_NOCNN=True
        export UTOOL_NOCNN True
        """
        #argv_orig = argv[:]
        for key, val in os.environ.items():
            key = key.upper()
            sentinal = 'UTOOL_'
            if key.startswith(sentinal):
                key = '--' + key[len(sentinal):].lower()
                new_argv = [key, val]
                if val.upper() in ['TRUE', 'FALSE', 'ON', 'OFF']:
                    # handled by get_argflag
                    continue
                argv = argv[:] + new_argv
                if debug:
                    print('argv.extend(new_argv=%r)' % (new_argv,))

        for argx, item in enumerate(argv):
            for argstr in argstr_list:
                if item == argstr:
                    if type_ is bool:
                        if debug:
                            print('[get_argval] ... argstr=%r' % (argstr,))
                            print('[get_argval] ... Found bool argx=%r' % (argx,))
                        arg_after = True
                        was_specified = True
                        break
                    if argx < len(argv):
                        if type_ is list:
                            # HACK FOR LIST. TODO INTEGRATE
                            if debug:
                                print('[get_argval] ... argstr=%r' % (argstr,))
                                print('[get_argval] ... Found noequal list argx=%r' % (argx,))
                            arg_after = parse_arglist_hack(argx, argv=argv)
                            if debug:
                                print('[get_argval] ... arg_after=%r' % (arg_after,))
                                print('argv=%r' % (argv,))
                            if smartcast:
                                arg_after = list(map(util_type.smart_cast2, arg_after))
                                if debug:
                                    print('[get_argval] ... smartcast arg_after=%r' % (arg_after,))
                        else:
                            if debug:
                                print('[get_argval] ... argstr=%r' % (argstr,))
                                print('[get_argval] ... Found type_=%r argx=%r' % (type_, argx,))
                            arg_after = argv[argx + 1]
                            if type_ is not None:
                                arg_after = util_type.try_cast(arg_after, type_)
                            elif smartcast:
                                arg_after = util_type.smart_cast2(arg_after)
                        if was_specified:
                            print('WARNING: argstr=%r already specified' % (argstr,))
                        was_specified = True
                        break
                elif item.startswith(argstr + '='):
                    val_after = ''.join(item.split('=')[1:])
                    if type_ is list:
                        # HACK FOR LIST. TODO INTEGRATE
                        if verbose:
                            print('[get_argval] ... Found equal list')
                        val_after_ = val_after.rstrip(']').lstrip('[')
                        if True:
                            # Hacker way to be less hacky about parsing lists
                            from utool import util_gridsearch
                            blocks = util_gridsearch.parse_nestings(val_after_)
                            sentinal = '##COM&&'
                            changed = [(block[0], block[1].replace(',', sentinal))
                                       if block[0] == 'nonNested' else block
                                       for block in blocks]
                            val_after2 = util_gridsearch.recombine_nestings(changed)
                            arg_after = val_after2.split(sentinal)
                        else:
                            arg_after = val_after_.split(',')
                        if smartcast:
                            arg_after = list(map(util_type.smart_cast2, arg_after))
                    else:
                        if type_ is None:
                            if smartcast:
                                arg_after = util_type.smart_cast2(val_after)
                            else:
                                arg_after = val_after
                        else:
                            arg_after = util_type.try_cast(val_after, type_)
                            if not isinstance(type_, six.string_types) and issubclass(type_, six.string_types):
                                if arg_after == 'None':
                                    # hack
                                    arg_after = None
                    if was_specified:
                        print('WARNING: argstr=%r already specified' % (argstr,))
                    was_specified = True
                    break
    except Exception as ex:
        import utool as ut
        ut.printex(ex, 'problem in arg_val', keys=['type_'])
        if ut.SUPER_STRICT:
            raise
        pass
    if not was_specified and pos is not None:
        varargs = get_cmdline_varargs(argv)
        if len(varargs) > pos:
            arg_after = varargs[pos]
            assert type_ is not list, 'list not handled yet'
            if type_ is not None:
                arg_after = util_type.try_cast(arg_after, type_)
            elif smartcast:
                arg_after = util_type.smart_cast2(arg_after)

    if verbose:
        print('[get_argval] ... Parsed arg_after=%r, was_specified=%r' % (arg_after, was_specified))
    if return_specified:
        return arg_after, was_specified
    else:
        return arg_after


@profile
def parse_cfgstr_list(cfgstr_list, smartcast=True, oldmode=True):
    r"""
    Parses a list of items in the format
    ['var1:val1', 'var2:val2', 'var3:val3']
    the '=' character can be used instead of the ':' character if desired

    TODO: see ut.parse_cfgstr3

    Args:
        cfgstr_list (list):

    Returns:
        dict: cfgdict

    Example:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_arg import *  # NOQA
        >>> import utool as ut
        >>> cfgstr_list = ['var1=val1', 'var2=1', 'var3=1.0', 'var4=None', 'var5=[1,2,3]', 'var6=(a,b,c)']
        >>> smartcast = True
        >>> cfgdict = parse_cfgstr_list(cfgstr_list, smartcast, oldmode=False)
        >>> result = ut.repr2(cfgdict, sorted_=True, newlines=False)
        >>> print(result)
        {'var1': 'val1', 'var2': 1, 'var3': 1.0, 'var4': None, 'var5': [1, 2, 3], 'var6': ('a', 'b', 'c')}

        {'var1': 'val1', 'var2': 1, 'var3': 1.0, 'var4': None}

        {'var4': None, 'var1': 'val1', 'var3': 1.0, 'var2': 1}
    """
    cfgdict = {}
    for item in cfgstr_list:
        if item == '':
            continue
        if oldmode:
            keyval_tup = item.replace('=', ':').split(':')
            assert len(keyval_tup) == 2, '[!] Invalid cfgitem=%r' % (item,)
            key, val = keyval_tup
        else:
            keyval_tup = item.split('=')
            if len(keyval_tup) == 1:
                # single specifications are interpeted as booleans
                key = keyval_tup[0]
                val = True
            else:
                assert len(keyval_tup) >= 2, '[!] Invalid cfgitem=%r' % (item,)
                key, val = keyval_tup[0], '='.join(keyval_tup[1:])
        if smartcast:
            val = util_type.smart_cast2(val)
        cfgdict[key] = val
    return cfgdict


def parse_arglist_hack(argx, argv=None):
    if argv is None:
        argv = sys.argv
    arglist = []
    #import utool as ut
    #ut.embed()
    for argx2 in range(argx + 1, len(argv)):
        listarg = argv[argx2]
        if listarg.startswith('-'):
            break
        else:
            arglist.append(listarg)
    return arglist


def get_arg_dict(argv=None, prefix_list=['--'], type_hints={}):
    r"""
    Yet another way for parsing args

    CommandLine:
        python -m utool.util_arg --exec-get_arg_dict
        python -m utool.util_arg --test-get_arg_dict

    Example:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_arg import *  # NOQA
        >>> import utool as ut
        >>> import shlex
        >>> argv = shlex.split('--test-show_name --name=IBEIS_PZ_0303 --db testdb3 --save "~/latex/crall-candidacy-2015/figures/IBEIS_PZ_0303.jpg" --dpath figures --caption="Shadowed"  --figsize=11,3 --no-figtitle -t foo bar baz biz --notitle')
        >>> arg_dict = ut.get_arg_dict(argv, prefix_list=['--', '-'], type_hints={'t': list})
        >>> result = ut.repr2(arg_dict, nl=1)
        >>> # verify results
        >>> print(result)
        {
            'caption': 'Shadowed',
            'db': 'testdb3',
            'dpath': 'figures',
            'figsize': '11,3',
            'name': 'IBEIS_PZ_0303',
            'no-figtitle': True,
            'notitle': True,
            'save': '~/latex/crall-candidacy-2015/figures/IBEIS_PZ_0303.jpg',
            't': ['foo', 'bar', 'baz', 'biz'],
            'test-show_name': True,
        }
    """
    if argv is None:
        argv = sys.argv
    arg_dict = {}

    def startswith_prefix(arg):
        return any([arg.startswith(prefix) for prefix in prefix_list])

    def argx_has_value(argv, argx):
        # Check if has a value
        if argv[argx].find('=') > -1:
            return True
        if argx + 1 < len(argv) and not startswith_prefix(argv[argx + 1]):
            return True
        return False

    def get_arg_value(argv, argx, argname):
        if argv[argx].find('=') > -1:
            return '='.join(argv[argx].split('=')[1:])
        else:
            type_ = type_hints.get(argname, None)
            if type_ is None:
                return argv[argx + 1]
            else:
                return parse_arglist_hack(argx, argv=argv)

    for argx in range(len(argv)):
        arg = argv[argx]
        for prefix in prefix_list:
            if arg.startswith(prefix):
                argname = arg[len(prefix):]
                if argx_has_value(argv, argx):
                    if arg.find('=') > -1:
                        argname = arg[len(prefix):arg.find('=')]
                    argvalue = get_arg_value(argv, argx, argname)
                    arg_dict[argname] = argvalue
                else:
                    arg_dict[argname] = True
                break
    return arg_dict

# Backwards Compatibility Aliases
get_arg  = get_argval
get_flag  = get_argflag


#def argv_flag(name, default, **kwargs):
#    if name.find('--') == 0:
#        name = name[2:]
#    if '--' + name in sys.argv and default is False:
#        return True
#    if '--no' + name in sys.argv and default is True:
#        return False
#    return default


# ---- OnTheFly argparse ^^^^
# ---- Documented argparse VVVV


def switch_sanataize(switch):
    if isinstance(switch, six.string_types):
        dest = switch.strip('-').replace('-', '_')
    else:
        if isinstance(switch, tuple):
            switch = switch
        elif isinstance(switch, list):
            switch = tuple(switch)
        dest = switch[0].strip('-').replace('-', '_')
    return dest, switch


class ArgumentParser2(object):
    """ Wrapper around argparse.ArgumentParser with convinence functions """
    def __init__(self, parser):
        self.parser = parser

    def add_arg(self, switch, *args, **kwargs):
        #print('[argparse2] add_arg(%r) ' % (switch,))
        if isinstance(switch, tuple):
            args = tuple(list(switch) + list(args))
            return self.parser.add_argument(*args, **kwargs)
        else:
            return self.parser.add_argument(switch, *args, **kwargs)

    def add_meta(self, switch, type, default=None, help='', **kwargs):
        #print('[argparse2] add_meta()')
        dest, switch = switch_sanataize(switch)
        self.add_arg(switch, metavar=dest, type=type, default=default, help=help, **kwargs)

    def add_flag(self, switch, default=False, **kwargs):
        #print('[argparse2] add_flag()')
        action = 'store_false' if default else 'store_true'
        dest, switch = switch_sanataize(switch)
        self.add_arg(switch, dest=dest, action=action, default=default, **kwargs)

    def add_int(self, switch, *args, **kwargs):
        self.add_meta(switch, util_type.fuzzy_int,  *args, **kwargs)

    def add_intlist(self, switch, *args, **kwargs):
        self.add_meta(switch, util_type.fuzzy_int,  *args, nargs='*', **kwargs)

    add_ints = add_intlist

    def add_strlist(self, switch, *args, **kwargs):
        self.add_meta(switch, str,  *args, nargs='*', **kwargs)

    add_strs = add_strlist

    def add_float(self, switch, *args, **kwargs):
        self.add_meta(switch, float, *args, **kwargs)

    def add_str(self, switch, *args, **kwargs):
        self.add_meta(switch, str, *args, **kwargs)

    def add_argument_group(self, *args, **kwargs):
        return ArgumentParser2(self.parser.add_argument_group(*args, **kwargs))


def autogen_argparse2(dpath_list):
    r"""

    FUNCTION IS NOT FULLY IMPLEMENTED CURRENTLY ONLY RETURNS
    LIST OF FLAGS THAT THE PROGRAM SILENTLY TAKES

    Example:
        >>> # DISABLE_DOCTEST
        >>> from utool.util_arg import *  # NOQA
        >>> import utool as ut
        >>> dpath_list = [
        ...     ut.truepath('~/code/utool/utool'),
        ...     ut.truepath('~/code/ibeis/ibeis'),
        ... ]
        >>> flagtups_list = autogen_argparse2(dpath_list)
        >>> flagtup_list_ = [ut.regex_replace('[)(\']','',tupstr) for tupstr in ut.flatten(flagtups_list)]
        >>> flagtup_list = ut.flatten([tupstr.split(',') for tupstr in flagtup_list_])
        >>> flagtup_set = set([tupstr.strip() for tupstr in flagtup_list if tupstr.find('=') == -1])
        >>> print('\n'.join(flagtup_set))
    """
    import utool as ut
    import parse
    include_patterns = ['*.py']
    regex_list = ['get_argflag', 'get_argval']
    recursive = True
    result = ut.grep(regex_list, recursive, dpath_list, include_patterns, verbose=True)
    (found_filestr_list, found_lines_list, found_lxs_list) = result
    # TODO: Check to see if in a comment block
    flagtups_list = []
    for found_lines in found_lines_list:
        flagtups = []
        for line in found_lines:
            line_ = ut.regex_replace('#.*', '', line)

            argval_parse_list = [
                '\'{flag}\' in sys.argv',
                'get_argval({flagtup}, type={type}, default={default})',
                'get_argval({flagtup}, {type}, default={default})',
                'get_argval({flagtup}, {type}, {default})',
                'get_argval({flagtup})',
            ]
            argflag_parse_list = [
                'get_argflag({flagtup})',
            ]
            def parse_pattern_list(parse_list, line):
                #result_list = []
                result = None
                for pattern in parse_list:
                    result = parse.parse('{_prefix}' + pattern, line_)
                    if result is not None:
                        break
                        #if len(result_list) > 1:
                        #    print('warning')
                        #result_list.append(result)
                return result
            val_result  = parse_pattern_list(argval_parse_list, line)
            flag_result = parse_pattern_list(argflag_parse_list, line)
            if flag_result is None and val_result is None:
                print('warning1')
            elif flag_result is not None and val_result is not None:
                print('warning2')
            else:
                result = flag_result if val_result is None else val_result
                flagtups.append(result['flagtup'])
        flagtups_list.append(flagtups)
    return flagtups_list


def make_argparse2(prog='Program', description='', *args, **kwargs):
    formatter_classes = [
        argparse.RawDescriptionHelpFormatter,
        argparse.RawTextHelpFormatter,
        argparse.ArgumentDefaultsHelpFormatter]
    parser = argparse.ArgumentParser(prog=prog,
                                     description=description,
                                     prefix_chars='+-',
                                     formatter_class=formatter_classes[2], *args,
                                     **kwargs)
    return ArgumentParser2(parser)


# Decorators which control program flow based on sys.argv
# the decorated function does not execute without its corresponding
# flag

def get_fpath_args(arglist_=None, pat='*'):
    import utool
    if arglist_ is None:
        arglist_ = sys.argv[1:]
    input_path_list = []
    for input_path in arglist_:
        input_path = utool.truepath(input_path)
        if os.path.isdir(input_path):
            input_path_list.extend(utool.glob(input_path, pat, recursive=False, with_dirs=False))
        else:
            input_path_list.append(input_path)
    return input_path_list


def argv_flag_dec(*argin, **kwargs):
    """
    Decorators which control program flow based on sys.argv
    the decorated function does not execute without its corresponding
    flag

    Kwargs:
        default, quiet, indent, default

    ReturnKwargs:
        alias_flags
    """
    kwargs = kwargs.copy()
    kwargs['default'] = kwargs.get('default', False)
    from utool import util_decor
    @util_decor.ignores_exc_tb(outer_wrapper=False)
    def wrap_argv_flag_dec(func):
        return __argv_flag_dec(func, **kwargs)

    assert len(argin) < 2, 'specify 0 or 1 args'

    if len(argin) == 1 and util_type.is_funclike(argin[0]):
        func = argin[0]
        return wrap_argv_flag_dec(func)
    else:
        return wrap_argv_flag_dec


def argv_flag_dec_true(func, **kwargs):
    return __argv_flag_dec(func, default=True, **kwargs)


def __argv_flag_dec(func, default=False, quiet=QUIET, indent=False):
    """
    Logic for controlling if a function gets called based on command line
    """
    from utool import util_decor
    flagname = meta_util_six.get_funcname(func)
    if flagname.find('no') == 0:
        flagname = flagname[2:]

    flags = (
        '--' + flagname.replace('_', '-'),
        '--' + flagname,
    )

    @util_decor.ignores_exc_tb(outer_wrapper=False)
    def GaurdWrapper(*args, **kwargs):
        from utool import util_print
        # FIXME: the --print-all is a hack
        default_ = kwargs.pop('default', default)
        alias_flags = kwargs.pop('alias_flags', [])
        is_flagged = (get_argflag(flags, default_) or
                      get_argflag('--print-all') or
                      any([get_argflag(_) for _ in alias_flags]))
        if flagname in kwargs:
            is_flagged = kwargs.pop(flagname)
        if is_flagged:
            func_label = flags[0].replace('--', '').replace('print-', '')
            # print('')
            print('\n+ --- ' + func_label + ' ___')
            use_indent = indent is not False
            if indent is True:
                indent_ = '[%s]' % func_label
            else:
                indent_ = indent
            with util_print.Indenter(indent_, enabled=use_indent):
                ret = func(*args, **kwargs)
            print('L ___ ' + func_label + '___\n')
            return ret
        else:
            PRINT_DISABLED_FLAGDEC = not get_argflag(
                '--noinform', help_='does not print disabled flag decorators')
            if not quiet and PRINT_DISABLED_FLAGDEC:
                #print('\n~~~ %s ~~~' % flag)
                print('~~~ %s ~~~' % flags[0])
    meta_util_six.set_funcname(GaurdWrapper, meta_util_six.get_funcname(func))
    return GaurdWrapper


@profile
def argparse_dict(default_dict_, lbl=None, verbose=None,
                  only_specified=False, force_keys={}, type_hint=None,
                  alias_dict={}):
    r"""
    Gets values for a dict based on the command line

    Args:
        default_dict_ (?):
        only_specified (bool): if True only returns keys that are specified on commandline. no defaults.

    Returns:
        dict_: dict_ -  a dictionary

    CommandLine:
        python -m utool.util_arg --test-argparse_dict
        python -m utool.util_arg --test-argparse_dict --foo=3
        python -m utool.util_arg --test-argparse_dict --flag1
        python -m utool.util_arg --test-argparse_dict --flag2
        python -m utool.util_arg --test-argparse_dict --noflag2
        python -m utool.util_arg --test-argparse_dict --thresh=43
        python -m utool.util_arg --test-argparse_dict --bins=-10
        python -m utool.util_arg --test-argparse_dict --bins=-10 --only-specified --helpx

    Example:
        >>> # DISABLE_DOCTEST
        >>> from utool.util_arg import *  # NOQA
        >>> import utool as ut
        >>> # build test data
        >>> default_dict_ = {
        ...    'bins': 8,
        ...    'foo': None,
        ...    'flag1': False,
        ...    'flag2': True,
        ...    'max': 0.2,
        ...    'neg': -5,
        ...    'thresh': -5.333,
        ... }
        >>> # execute function
        >>> only_specified = ut.get_argflag('--only-specified')
        >>> dict_ = argparse_dict(default_dict_, only_specified=only_specified)
        >>> # verify results
        >>> result = ut.repr4(dict_, sorted_=True)
        >>> print(result)
    """
    if verbose is None:
        verbose = VERBOSE_ARGPARSE
    def make_argstrs(key, prefix_list):
        for prefix in prefix_list:
            yield prefix + key
            yield prefix + key.replace('-', '_')
            yield prefix + key.replace('_', '-')

    def get_dictkey_cmdline_val(key, default, type_hint):
        # see if the user gave a commandline value for this dict key
        defaulttype_ = None if default is None else type(default)
        if type_hint is None:
            type_ = defaulttype_
        elif isinstance(type_hint, dict):
            type_ = type_hint.get(key, defaulttype_)
        elif isinstance(type_hint, type):
            type_ = type_hint
        else:
            raise NotImplementedError('Unknown type of type_hint=%r' % (type_hint,))
        was_specified = False
        if isinstance(default, bool):
            val = default
            if default is True:
                falsekeys = list(set(make_argstrs(key, ['--no', '--no-'])))
                notval, was_specified = get_argflag(falsekeys, return_specified=True)
                val = not notval
                if not was_specified:
                    truekeys = list(set(make_argstrs(key, ['--'])))
                    val_, was_specified = get_argflag(truekeys, return_specified=True)
                    if was_specified:
                        val = val_
            elif default is False:
                truekeys = list(set(make_argstrs(key, ['--'])))
                val, was_specified = get_argflag(truekeys, return_specified=True)
        else:
            argtup = list(set(make_argstrs(key, ['--'])))
            #if key == 'species':
            #    import utool as ut
            #    ut.embed()
            val, was_specified = get_argval(argtup, type_=type_,
                                            default=default,
                                            return_specified=True)
        return val, was_specified

    dict_  = {}
    num_specified = 0
    for key, default in six.iteritems(default_dict_):
        val, was_specified = get_dictkey_cmdline_val(key, default, type_hint)
        if not was_specified:
            alias_keys = meta_util_iter.ensure_iterable(alias_dict.get(key, []))
            for alias_key in alias_keys:
                val, was_specified = get_dictkey_cmdline_val(alias_key, default,
                                                             type_hint)
                if was_specified:
                    break
        if VERBOSE_ARGPARSE:
            if was_specified:
                num_specified += 1
                print('[argparse_dict] Specified key=%r, val=%r' % (key, val))
        #if key == 'foo':
        #    import utool as ut
        #    ut.embed()
        if not only_specified or was_specified or key in force_keys:
            dict_[key] = val
    if VERBOSE_ARGPARSE:
        print('[argparse_dict] num_specified = %r' % (num_specified,))
        print('[argparse_dict] force_keys = %r' % (force_keys,))
    #dict_ = {key: get_dictkey_cmdline_val(key, default) for key, default in
    #six.iteritems(default_dict_)}

    if verbose:
        for key in dict_:
            if dict_[key] != default_dict_[key]:
                print('[argparse_dict] GOT ARGUMENT: cfgdict[%r] = %r' % (key, dict_[key]))

    do_helpx = get_argflag('--helpx',
                           help_='Specifies that argparse_dict should print help and quit')

    if get_argflag(('--help', '--help2')) or do_helpx:
        import utool as ut
        print('COMMAND LINE IS ACCEPTING THESE PARAMS WITH DEFAULTS:')
        if lbl is not None:
            print(lbl)
        #print(ut.align(ut.repr4(dict_, sorted_=True), ':'))
        print(ut.align(ut.repr4(default_dict_, sorted_=True), ':'))
        if do_helpx:
            sys.exit(1)
    return dict_


def get_argv_tail(scriptname, prefer_main=None, argv=None):
    r"""
    gets the rest of the arguments after a script has been invoked hack.
    accounts for python -m scripts.

    Args:
        scriptname (str):

    CommandLine:
        python -m utool.util_arg --test-get_argv_tail

    Example:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_arg import *  # NOQA
        >>> import utool as ut
        >>> from os.path import relpath, dirname
        >>> scriptname = 'utool.util_arg'
        >>> prefer_main = False
        >>> argv=['python', '-m', 'utool.util_arg', '--test-get_argv_tail']
        >>> tail = get_argv_tail(scriptname, prefer_main, argv)
        >>> # hack
        >>> tail[0] = ut.ensure_unixslash(relpath(tail[0], dirname(dirname(ut.__file__))))
        >>> result = ut.repr2(tail)
        >>> print(result)
        ['utool/util_arg.py', '--test-get_argv_tail']

    Example:
        >>> # ENABLE_DOCTEST
        >>> from utool.util_arg import *  # NOQA
        >>> import utool as ut
        >>> from os.path import relpath, dirname
        >>> scriptname = 'utprof.py'
        >>> prefer_main = True
        >>> argv=['utprof.py', '-m', 'utool', '--tf', 'get_argv_tail']
        >>> tail = get_argv_tail(scriptname, prefer_main, argv)
        >>> # hack
        >>> tail[0] = ut.ensure_unixslash(relpath(tail[0], dirname(dirname(ut.__file__))))
        >>> result = ut.repr2(tail)
        >>> print(result)
        ['utool/__main__.py', '--tf', 'get_argv_tail']
    """
    if argv is None:
        argv = sys.argv
    import utool as ut
    modname = ut.get_argval('-m', help_='specify module name to profile', argv=argv)
    if modname is not None:
        # hack to account for -m scripts
        modpath = ut.get_modpath(modname, prefer_main=prefer_main)
        argvx = argv.index(modname) + 1
        argv_tail = [modpath] + argv[argvx:]
    else:
        try:
            argvx = argv.index(scriptname)
        except ValueError:
            for argvx, arg in enumerate(argv):
                # HACK
                if scriptname in arg:
                    break
        argv_tail = argv[(argvx + 1):]
    return argv_tail


def get_cmdline_varargs(argv=None):
    """
    Returns positional args specified directly after the scriptname
    and before any args starting with '-' on the commandline.
    """
    if argv is None:
        argv = sys.argv
    scriptname = argv[0]
    if scriptname == '':
        # python invoked by iteself
        pos_start = 0
        pos_end = 0
    else:
        pos_start = pos_end = 1
        for idx in range(pos_start, len(argv)):
            if argv[idx].startswith('-'):
                pos_end = idx
                break
        else:
            pos_end = len(argv)
    cmdline_varargs = argv[pos_start:pos_end]
    return cmdline_varargs


get_varargs = get_cmdline_varargs


# alias
parse_dict_from_argv = argparse_dict
get_dict_vals_from_commandline = argparse_dict

# Do I like these more?
# varargs = get_varargs
argval = get_argval
argflag = get_argflag


def argval(key, default=None, type=None, smartcast=True, return_exists=False,
           argv=None):
    """
    alias for get_argval

    Ignore:
        >>> # ENABLE_DOCTEST
        >>> import utool as ut
        >>> import sys
        >>> argv = ['--aids=[1,2,3]']
        >>> value = ut.argval('--aids', default=[1, 2], argv=argv)
        >>> assert isinstance(value, list)
        >>> value2 = ut.argval('--aids', smartcast=False, argv=argv)
        >>> assert isinstance(value2, str)
        >>> value2 = ut.argval('--aids', smartcast=True, argv=argv)
        >>> assert isinstance(value2, list)
    """
    defaultable_types = (tuple, list, int, float)
    if type is None and isinstance(default, defaultable_types):
        type = builtins.type(default)
    return get_argval(key, type_=type, default=default,
                      return_was_specified=return_exists, smartcast=smartcast,
                      argv=argv)


VERBOSE_ARGPARSE = get_argflag(
    ('--verbose-argparse', '--verb-argparse', '--verb-arg', '--verbarg'),
    help_='debug util_arg')


#if __name__ == '__main__':
#    """
#    CommandLine:
#        python utool/util_arg.py --test-autogen_argparse2
#    """
#    from utool import util_tests
#    util_tests.doctest_funcs([autogen_argparse2])
if __name__ == '__main__':
    """
    CommandLine:
        python -m utool.util_arg
        python -m utool.util_arg --allexamples
        python -m utool.util_arg --allexamples --noface --nosrc
    """
    import multiprocessing
    multiprocessing.freeze_support()  # for win32
    import utool as ut  # NOQA
    ut.doctest_funcs()
